/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev.h"
#include <string>

static bool on_delete_event (GtkWidget*, GdkEvent*, gpointer);
static void on_destroy (GtkObject*, gpointer);

// The magic all begins here...
int main (int argc, 
          char *argv[])
{
  if (argc > 1)
  {
    if (string(argv[1]) == "--help" || string(argv[1]) == "-?")
    {
      cout << "For help information, run 'man openldev'.\n";
      return EXIT_SUCCESS;
    }
    else if (string(argv[1]) == "--version")
    {
      cout << "Open Linux Developer 0.5.0 (Chinstrap)\nCopyright (C) 2004-2005 Andrew Krause\n";
      return EXIT_SUCCESS;
    }
  }
  
  gtk_init (&argc, &argv);
  gnome_program_init ("openldev", VERSION, LIBGNOMEUI_MODULE, argc, argv, GNOME_PARAM_NONE);
  
  GtkWidget *window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  OpenLDev *win = openldev_new_with_window (window);
  string icon = string(INSTALL_PREFIX) + "/share/pixmaps/openldev/openldev24.png";
  gtk_window_set_icon_from_file (GTK_WINDOW(window), icon.c_str(), NULL);

  g_signal_connect (G_OBJECT(window), "delete-event",
                    G_CALLBACK(on_delete_event), win);
  g_signal_connect (G_OBJECT(window), "destroy",
                    G_CALLBACK(on_destroy), NULL);
  
  gtk_widget_show (window);
  gtk_main();
	
  return EXIT_SUCCESS;
}

static bool on_delete_event (GtkWidget *widget, 
                             GdkEvent *event, 
                             gpointer data)
{
  OpenLDev *win = (OpenLDev*) data;
  
  bool is_modified = false;
  for (int i=0; i < win->notebook->tab_count; i++)
  {
    string label = gtk_label_get_text (GTK_LABEL(win->notebook->tabs[i]->label));
    if (label.find("*", 0) != string::npos)
      is_modified = true;
  }
  
  if (!is_modified)
    return false;
  else if (openldev_file_manager_question("You have unsaved files! Are you.\nsure you want to close?"))
    return false;
  
  return true;
}

static void on_destroy (GtkObject *object,
                        gpointer data)
{
  gtk_main_quit();
}
