/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "makefile-editor.h"

void makefile_editor_populate_makefiles (MakefileEditor*, vector<string>&);
void makefile_editor_sort_makefile (Makefile*, int, int);
void makefile_editor_error (const char *);

void makefile_editor_makefile_changed (GtkComboBox*, gpointer);
void makefile_editor_selection_changed (GtkTreeSelection*, gpointer);
void makefile_editor_edited (GtkCellRendererText*, gchar*, gchar*, gpointer);

void makefile_editor_new_element (GtkButton*, gpointer);
void makefile_editor_close (GtkButton*, gpointer);
void makefile_editor_move_up (GtkButton*, gpointer);
void makefile_editor_move_down (GtkButton*, gpointer);
void makefile_editor_add (GtkButton*, gpointer);
void makefile_editor_delete (GtkButton*, gpointer);

MakefileEditor* makefile_editor_new (const char *file)
{
  MakefileEditor *me = new MakefileEditor;
  vector<string> makefile_list;
  
  if (g_file_test (file, G_FILE_TEST_EXISTS))
  {
    me->project = openldev_project_settings_new (file);
    makefile_list = openldev_project_settings_get_makefile_list (me->project);
  }
  else
    makefile_editor_error("Project file does not exist!");
  
  string gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/makefileeditor.glade";
  GladeXML *xml = glade_xml_new(gladefile.c_str(), NULL, NULL);

  me->window    = glade_xml_get_widget (xml, "window");
  me->makefiles = glade_xml_get_widget (xml, "makefiles");
  me->treeview  = glade_xml_get_widget (xml, "treeview");
  me->b_new     = glade_xml_get_widget (xml, "new");
  me->b_add     = glade_xml_get_widget (xml, "add");
  me->b_delete  = glade_xml_get_widget (xml, "delete");
  me->b_up      = glade_xml_get_widget (xml, "up");
  me->b_down    = glade_xml_get_widget (xml, "down");
  me->b_help    = glade_xml_get_widget (xml, "help");
  me->b_close   = glade_xml_get_widget (xml, "close");
  
  g_signal_connect (G_OBJECT(me->b_close), "clicked",
                    G_CALLBACK(makefile_editor_close), (gpointer)me);
  g_signal_connect (G_OBJECT(me->b_up), "clicked",
                    G_CALLBACK(makefile_editor_move_up), (gpointer)me);
  g_signal_connect (G_OBJECT(me->b_down), "clicked",
                    G_CALLBACK(makefile_editor_move_down), (gpointer)me);
  g_signal_connect (G_OBJECT(me->b_add), "clicked",
                    G_CALLBACK(makefile_editor_add), (gpointer)me);
  g_signal_connect (G_OBJECT(me->b_delete), "clicked",
                    G_CALLBACK(makefile_editor_delete), (gpointer)me);
  g_signal_connect (G_OBJECT(me->b_new), "clicked",
                    G_CALLBACK(makefile_editor_new_element), (gpointer)me);
  g_signal_connect (G_OBJECT(me->makefiles), "changed",
                    G_CALLBACK(makefile_editor_makefile_changed), (gpointer)me);
  
  gtk_widget_hide (me->b_help);
  gtk_tree_selection_set_mode (gtk_tree_view_get_selection(GTK_TREE_VIEW(me->treeview)),
                               GTK_SELECTION_SINGLE);
  makefile_editor_populate_makefiles (me, makefile_list);
  gtk_combo_box_set_active (GTK_COMBO_BOX(me->makefiles), 0);
  
  makefile_list.clear();
  
  return me;
}

void makefile_editor_close (GtkButton *close, 
                            gpointer data)
{ gtk_widget_hide (((MakefileEditor*)data)->window);
}

void makefile_editor_add (GtkButton *b_add, 
                          gpointer data)
{
  MakefileEditor *me = (MakefileEditor*)data;
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(me->treeview));
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(me->treeview));
  GtkTreeIter current, new_iter;
  GtkTreePath *treepath;
  string path;
  
  gtk_tree_selection_get_selected (selection, &model, &current);
  treepath = gtk_tree_model_get_path (model, &current);
  path = gtk_tree_path_to_string(gtk_tree_model_get_path (model, &current));
  int location = atol (path.c_str());
  
  if (me->mf->names[location].find("SOURCES", 0) != string::npos ||
      me->mf->names[location].find("HEADERS", 0) != string::npos)
  {
    gchar *file;
    GtkTreeIter comboiter;
    GtkTreeModel *combomodel = gtk_combo_box_get_model (GTK_COMBO_BOX(me->makefiles));
    gtk_combo_box_get_active_iter (GTK_COMBO_BOX(me->makefiles), &comboiter);
    gtk_tree_model_get (combomodel, &comboiter, 0, &file, -1);
  
    string folder = me->project->location.substr (0, me->project->location.length()-1) + file;
    folder.erase (folder.find("Makefile.am", 0), 11);
    
    GtkWidget *dialog = gtk_file_chooser_dialog_new ("Choose a file ...",
                            GTK_WINDOW(me->window), GTK_FILE_CHOOSER_ACTION_OPEN,
                            GTK_STOCK_ADD, GTK_RESPONSE_CANCEL,
                            GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog), folder.c_str());
    
    int result = gtk_dialog_run (GTK_DIALOG(dialog));
    switch (result)
    {
      // If OK is pressed
      case (GTK_RESPONSE_OK):
      {
        string filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(dialog));
        gtk_widget_destroy (dialog);
      
        if (filename.find(folder, 0) != string::npos)
        {
          filename.erase (0, folder.length());
          if (filename.find("/", 0) == string::npos)
          {
            gtk_tree_store_insert (GTK_TREE_STORE(model), &new_iter, &current, 1000);
            gtk_tree_store_set (GTK_TREE_STORE(model), &new_iter, 0, filename.c_str(), -1);
            me->mf->attributes[location]->attr.push_back (filename);
          }
          else
            makefile_editor_error ("Please choose a file in the same\nfolder as the chosen makefile!");
        }
        else
          makefile_editor_error ("Please choose a file in the same\nfolder as the chosen makefile!");
        
        filename = "\0";
        break;
      }
      default:
        gtk_widget_destroy (dialog);
        break;
    }
    
    file = "\0";
  }
  else
  {
    string script = openldev_input_dialog_run ("Attribute: ", "Enter Text to Append");
    
    if (script != "EMPTY" && !script.empty())
    {
      gtk_tree_store_insert (GTK_TREE_STORE(model), &new_iter, &current, 1000);
      gtk_tree_store_set (GTK_TREE_STORE(model), &new_iter, 0, script.c_str(), -1);
      me->mf->attributes[location]->attr.push_back (script);
    }
    script = "\0";
  }
  
  openldev_makefile_parser_write (me->mf);
}

void makefile_editor_delete (GtkButton *b_delete, 
                             gpointer data)
{
  MakefileEditor *me = (MakefileEditor*)data;
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(me->treeview));
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(me->treeview));
  GtkTreeIter current;
  GtkTreePath *treepath;
  string path;
  
  gtk_tree_selection_get_selected (selection, &model, &current);
  treepath = gtk_tree_model_get_path (model, &current);
  path = gtk_tree_path_to_string(gtk_tree_model_get_path (model, &current));
  gtk_tree_store_remove (GTK_TREE_STORE(model), &current);
  
  if (path.find(":") != string::npos)
  {
    int parent = atol (path.substr(0, path.find(":", 0)).c_str());
    path.erase (0, path.find(":", 0) + 1);
    int child = atol (path.c_str());
    
    if (child > 0)
    {
      vector<string>::iterator iter = me->mf->attributes[parent]->attr.begin();
      for (int i = 0; i < child; i++)
        iter++;
      me->mf->attributes[parent]->attr.erase(iter);
    }
    else
      me->mf->attributes[parent]->attr.clear();
  }
  else
  {
    int location = atol (path.c_str());
    vector<int>::iterator iter_int = me->mf->types.begin();
    vector<string>::iterator iter_str = me->mf->names.begin();
    vector<MakefileAttr*>::iterator iter_ma = me->mf->attributes.begin();
    
    for (int j = 0; j < location; j++)
    {
      iter_int++;
      iter_str++;
      iter_ma++;
    }
    
    me->mf->types.erase(iter_int);
    me->mf->names.erase(iter_str);
    me->mf->attributes.erase(iter_ma);
  }
  
  openldev_makefile_parser_write (me->mf);
}

void makefile_editor_new_element (GtkButton *b_add, 
                                  gpointer data)
{ makefile_editor_error ("This function has not yet been implemented! To\nadd a base element, you must edit the Makefile.am\ndirectly.");
}

void makefile_editor_sort_makefile (Makefile *mf,
                                    int first,
                                    int second)
{
  int temp_int;
  MakefileAttr *temp_ma;
  string temp_str;
    
  temp_int = mf->types[first];
  mf->types[first] = mf->types[second];
  mf->types[second] = temp_int;
    
  temp_ma = mf->attributes[first];
  mf->attributes[first] = mf->attributes[second];
  mf->attributes[second] = temp_ma;
    
  temp_str = mf->names[first];
  mf->names[first] = mf->names[second];
  mf->names[second] = temp_str;
  temp_str = "\0";
}

void makefile_editor_move_up (GtkButton *up, 
                              gpointer data)
{
  MakefileEditor *me = (MakefileEditor*)data;
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(me->treeview));
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(me->treeview));
  GtkTreeIter current, prev;
  GtkTreePath *treepath;
  string path;
  
  gtk_tree_selection_get_selected (selection, &model, &current);
  treepath = gtk_tree_model_get_path (model, &current);
  path = gtk_tree_path_to_string(gtk_tree_model_get_path (model, &current));
  
  if (path.find(":") != string::npos)
  {
    int parent = atol (path.substr(0, path.find(":", 0)).c_str());
    path.erase (0, path.find(":", 0) + 1);
    int child = atol (path.c_str());
    string temp;    
    
    temp = me->mf->attributes[parent]->attr[child-1];
    me->mf->attributes[parent]->attr[child-1] = me->mf->attributes[parent]->attr[child];
    me->mf->attributes[parent]->attr[child] = temp;
    temp = "\0";
  }
  else
  {
    int location = atol (path.c_str());
    makefile_editor_sort_makefile (me->mf, location, location-1);
  }
  
  openldev_makefile_parser_write (me->mf);
  gtk_tree_path_prev (treepath);
  gtk_tree_model_get_iter (model, &prev, treepath);
  gtk_tree_store_swap (GTK_TREE_STORE(model), &prev, &current);
  makefile_editor_selection_changed (gtk_tree_view_get_selection(GTK_TREE_VIEW(me->treeview)), data);
  path = "\0";
}

void makefile_editor_move_down (GtkButton *down, 
                                gpointer data)
{
  MakefileEditor *me = (MakefileEditor*)data;
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(me->treeview));
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(me->treeview));
  GtkTreeIter current, next;
  GtkTreePath *treepath;
  string path;
  
  gtk_tree_selection_get_selected (selection, &model, &current);
  treepath = gtk_tree_model_get_path (model, &current);
  path = gtk_tree_path_to_string(gtk_tree_model_get_path (model, &current));
  
  if (path.find(":") != string::npos)
  {
    int parent = atol (path.substr(0, path.find(":", 0)).c_str());
    path.erase (0, path.find(":", 0) + 1);
    int child = atol (path.c_str());
    string temp;    
    
    temp = me->mf->attributes[parent]->attr[child+1];
    me->mf->attributes[parent]->attr[child+1] = me->mf->attributes[parent]->attr[child];
    me->mf->attributes[parent]->attr[child] = temp;
    temp = "\0";
  }
  else
  {
    int location = atol (path.c_str());
    makefile_editor_sort_makefile (me->mf, location, location+1);
  }
  
  openldev_makefile_parser_write (me->mf);
  gtk_tree_path_next (treepath);
  gtk_tree_model_get_iter (model, &next, treepath);
  gtk_tree_store_swap (GTK_TREE_STORE(model), &next, &current);
  makefile_editor_selection_changed (gtk_tree_view_get_selection(GTK_TREE_VIEW(me->treeview)), data);
  path = "\0";
}

void makefile_editor_makefile_changed (GtkComboBox *combobox,
                                       gpointer data)
{
  MakefileEditor *me = (MakefileEditor*)data;
  GtkTreeIter iter, child;
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  gchar *file;
  string makefile;
  GtkTreeModel *model = gtk_combo_box_get_model (combobox);
  
  gtk_combo_box_get_active_iter (combobox, &iter);
  gtk_tree_model_get (model, &iter, 0, &file, -1);
  makefile = me->project->location.substr (0, me->project->location.length()-1) + file;
  g_object_unref (model);
  delete me->mf;
  me->mf = openldev_makefile_parser_new (makefile.c_str());
  
  GtkTreeStore *store = gtk_tree_store_new (1, G_TYPE_STRING);
  gtk_tree_view_set_model (GTK_TREE_VIEW(me->treeview), GTK_TREE_MODEL(store));
  g_signal_connect (G_OBJECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(me->treeview))), "changed",
                    G_CALLBACK(makefile_editor_selection_changed), (gpointer)me);
  
  while (g_list_length(gtk_tree_view_get_columns(GTK_TREE_VIEW(me->treeview))) > 0)
    gtk_tree_view_remove_column(GTK_TREE_VIEW(me->treeview),
      gtk_tree_view_get_column(GTK_TREE_VIEW(me->treeview), 0));

  for (unsigned int i=0; i < me->mf->names.size(); i++)
  {
    gtk_tree_store_append (store, &iter, NULL);
    gtk_tree_store_set (store, &iter, 0, me->mf->names[i].c_str(), -1);

    for (unsigned int j=0; j < me->mf->attributes[i]->attr.size(); j++)
    {
      gtk_tree_store_append (store, &child, &iter);
      gtk_tree_store_set (store, &child, 0, me->mf->attributes[i]->attr[j].c_str(), -1);
    }
  }
  
  g_object_unref (store);
  renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes ("Elements", renderer, "text", 0, NULL);
	g_object_set (renderer, "editable", true, NULL);
  g_signal_connect(G_OBJECT(renderer), "edited", G_CALLBACK(makefile_editor_edited), (gpointer)me);
  gtk_tree_view_append_column (GTK_TREE_VIEW(me->treeview), column);
  gtk_tree_view_expand_all (GTK_TREE_VIEW(me->treeview));
  
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(me->treeview));
  gtk_tree_model_get_iter (gtk_tree_view_get_model(GTK_TREE_VIEW(me->treeview)),
                           &iter, gtk_tree_path_new_from_string("0"));
  gtk_tree_selection_select_iter (selection, &iter);
}

void makefile_editor_populate_makefiles (MakefileEditor *me,
                                         vector<string> &makefile_list)
{
  GtkTreeModel *model= GTK_TREE_MODEL(gtk_list_store_new(1, G_TYPE_STRING));
  GtkTreeIter iter;
  GtkCellRenderer *renderer;
  
  for (unsigned int j=0; j < makefile_list.size()/2; j++)
  {
    string makefile = makefile_list[j];
    makefile_list[j] = makefile_list[makefile_list.size()-j-1];
    makefile_list[makefile_list.size()-j-1] = makefile;
  }
  
	
  for (unsigned int i=0; i < makefile_list.size(); i++) 
  {
    makefile_list[i].erase (0, me->project->location.length()-1);
    gtk_list_store_append(GTK_LIST_STORE(model), &iter);
    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, makefile_list[i].c_str(), -1);
  }
  
  gtk_combo_box_set_model(GTK_COMBO_BOX(me->makefiles), model);
  renderer = gtk_cell_renderer_text_new();
  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT(me->makefiles), renderer, TRUE);
  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT(me->makefiles), renderer, "text", 0, NULL);
  g_object_unref(model);
}

void makefile_editor_selection_changed (GtkTreeSelection *selection,
                                        gpointer data)
{
  MakefileEditor *me = (MakefileEditor*)data;

  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(me->treeview));
  GtkTreeIter current;
  GtkTreePath *current_path;
  string path;
  
  gtk_tree_selection_get_selected (selection, &model, &current);
  
  if (gtk_tree_store_iter_is_valid (GTK_TREE_STORE(model), &current))
  {
    current_path = gtk_tree_model_get_path (model, &current);
    path = gtk_tree_path_to_string (current_path);
  
    while (path.find(":", 0) != string::npos)
      path.erase (0, path.find(":", 0) + 1);
  
    if (gtk_tree_model_iter_n_children(model, &current) == 0)
      gtk_widget_set_sensitive (me->b_add, false);
    else
      gtk_widget_set_sensitive (me->b_add, true);

    if (path == "0")
      gtk_widget_set_sensitive (me->b_up, false);
    else
      gtk_widget_set_sensitive (me->b_up, true);
  
    if (!gtk_tree_model_iter_next(model, &current))
      gtk_widget_set_sensitive (me->b_down, false);
    else
      gtk_widget_set_sensitive (me->b_down, true);
    
    gtk_widget_set_sensitive (me->b_delete, true);
  }
  else
  {
    gtk_widget_set_sensitive (me->b_add, false);
    gtk_widget_set_sensitive (me->b_up, false);
    gtk_widget_set_sensitive (me->b_down, false);
    gtk_widget_set_sensitive (me->b_delete, false);
  }
}

void makefile_editor_edited (GtkCellRendererText *text, 
                             gchar *path_string, 
                             gchar *new_text, 
                             gpointer data)
{
  MakefileEditor *me = (MakefileEditor*) data;

  GtkTreeIter iter;
  GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
  string str = gtk_tree_path_to_string (path);
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(me->treeview));
  gtk_tree_model_get_iter (model, &iter, path);
  gtk_tree_store_set (GTK_TREE_STORE(model), &iter, 0, new_text, -1 );
  
  if (str.find(":", 0) != string::npos)
  {
    int parent = atol (str.substr(0, str.find(":", 0)).c_str());
    str.erase (0, str.find(":", 0) + 1);
    int child = atol (str.c_str());
    me->mf->attributes[parent]->attr[child] = new_text;
  }
  else
  {
    int location = atol (str.c_str());
    me->mf->names[location] = new_text;
  }
  
  makefile_editor_selection_changed (gtk_tree_view_get_selection(GTK_TREE_VIEW(me->treeview)), data);
  openldev_makefile_parser_write (me->mf);
}

// Throw an error dialog with the specified message
void makefile_editor_error (const char *message)
{
  GtkWidget *dialog;
  dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, 
                                   GTK_BUTTONS_CLOSE, message);
  int result = gtk_dialog_run (GTK_DIALOG(dialog));
  
  switch (result)
  {
    default:
      gtk_widget_destroy(dialog);
      message = "\0";
  }
}
