/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-build.h"

// Run the autogen.sh command if a project is currently open.
void openldev_build_auto_generate (GtkMenuItem *menuitem,
                                   gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev->project->name.empty())
    openldev_error ("Please open or create\na project first!");
  else 
  {
    gtk_notebook_set_current_page (GTK_NOTEBOOK(openldev->output->notebook), 0);
    chdir (openldev->project->location.c_str());
    
    string command = openldev->ptemplate->autogen + " 2>&1";
    openldev_output_build_run_command (openldev->output->build, 
                                       command.c_str(), 
                                       openldev->env);
    command = "\0";
  }
}

// If a project is open, build it. Run autogen.sh and then build again if
// Makefiles are not found. All of the files are first saved.
void openldev_build_project (GtkMenuItem *menuitem,
                             gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev->project->name.empty())
    openldev_error ("Please open or create\na project first!");
  else 
  {
    gtk_notebook_set_current_page(GTK_NOTEBOOK(openldev->output->notebook), 0);
    openldev_file_manager_save_all_files (openldev->notebook);
    chdir (openldev->project->location.c_str());
    
    string command = openldev->ptemplate->build + " 2>&1";
    if (!openldev_output_build_run_command(openldev->output->build, 
                                           command.c_str(), 
                                           openldev->env))
    {
      openldev_menu_build_auto_generate (openldev);
      openldev_output_build_run_command (openldev->output->build, 
                                         command.c_str(), 
                                         openldev->env);
    }
    command = "\0";
  }
}

// Clean all of the sources, build files, object files and those files
// created by configure
void openldev_build_clean (GtkMenuItem *menuitem,
                           gpointer data)
{  
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev->project->name.empty())
    openldev_error ("Please open or create\na project first!");
  else 
  {
    gtk_notebook_set_current_page(GTK_NOTEBOOK(openldev->output->notebook), 0);
    chdir (openldev->project->location.c_str());
    
    string command = string(openldev->ptemplate->clean) + " 2>&1";
    openldev_output_build_run_command (openldev->output->build, 
                                       command.c_str(), 
                                       openldev->env);
    command = "\0";
  }
}

// Compile current file if a file and a project is open. The function then
// creates the compile string from the project language & runs the command
void openldev_build_compile (GtkMenuItem *menuitem,
                             gpointer data)
{  
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (!openldev_file_manager_get_file_open(openldev->notebook))
    openldev_error ("Please open or create\na file first!");
  else 
  {
    gtk_notebook_set_current_page (GTK_NOTEBOOK(openldev->output->notebook), 0);
    string command = openldev->ptemplate->compile,
           fn = openldev_file_manager_get_current_filename (openldev->notebook);
    
    int loc = command.find("{SOURCE}", 0);

    command.erase (loc, 8);
    command.insert (loc, fn);
  
    loc = command.find ("{OBJECT}", 0);
    command.erase (loc, 8);
    fn.erase (fn.find(".", 0), fn.length()-fn.find(".", 0));
    command.insert (loc, fn);
    command += ".o";
    
    string location = openldev->project->location + "src/";
    chdir (openldev->project->location.c_str());

    command += " 2>&1";
    openldev_output_build_run_command (openldev->output->build, 
                                       command.c_str(), 
                                       openldev->env);
    command = "\0";
    location = "\0";
    fn = "\0";
  }
}

// If a project is open, run the ./configure command in the project directory
void openldev_build_configure (GtkMenuItem *menuitem,
                               gpointer data)
{  
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev->project->name.empty())
    openldev_error ("Please open or create\na project first!");
  else 
  {
    string parameters = openldev_input_dialog_run ("Parameters:", "Configure Parameters");
    if (parameters != "EMPTY")
    {
      gtk_notebook_set_current_page (GTK_NOTEBOOK(openldev->output->notebook), 0);
      chdir (openldev->project->location.c_str());
    
      string command = openldev->ptemplate->configure + " " + parameters + " 2>&1";
      openldev_output_build_run_command (openldev->output->build, 
                                         command.c_str(), 
                                         openldev->env);
      command = "\0";
      parameters = "\0";
    }
  }
}

// If a project is open, run the executable in the terminal specified
// by the user 
void openldev_build_execute (GtkMenuItem *menuitem,
                             gpointer data)
{  
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev->project->name.empty())
    openldev_error ("Please open or create\na project first!");
  else 
  {
    string command = openldev->project->location + "src/" + openldev->project->name + "\n";
    gtk_notebook_set_current_page (GTK_NOTEBOOK(openldev->output->notebook), 2);
    openldev_output_terminal_run_command (openldev->output->terminal, command.c_str());
    command = "\0";
  }
}

// Stop the current compilation, clean, configure, autogen or build
void openldev_build_stop (GtkMenuItem *menuitem,
                          gpointer data)
{  
  OpenLDev *openldev = (OpenLDev*) data;
  openldev_output_build_stop_build (openldev->output->build);
}
