/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-environment.h"

string openldev_env_dialog_get_color_string (GdkColor&);

EnvironmentDialog* openldev_env_dialog_new ()
{
  EnvironmentDialog *env = new EnvironmentDialog();
  
  PangoFontDescription *fd = pango_font_description_from_string("Sans Bold 10");
  string gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/environment.glade";
  GladeXML *xml = glade_xml_new(gladefile.c_str(), NULL, NULL);

  env->autoindent =     glade_xml_get_widget (xml, "auto_indent");
  env->autosave =       glade_xml_get_widget (xml, "auto_save");
  env->bracket =        glade_xml_get_widget (xml, "highlight_brackets");
  env->cancel =         glade_xml_get_widget (xml, "button_cancel");
  env->dialog =         glade_xml_get_widget (xml, "dialog");
  env->error =          glade_xml_get_widget (xml, "error_color");
  env->fontbutton =     glade_xml_get_widget (xml, "fontbutton");
  env->load_on_launch = glade_xml_get_widget (xml, "load_on_launch");
  env->ok =             glade_xml_get_widget (xml, "button_ok");
  env->replace_tabs =   glade_xml_get_widget (xml, "insert_spaces");
  env->right_margin =   glade_xml_get_widget (xml, "display_margin");
  env->split_words =    glade_xml_get_widget (xml, "split_words");
  env->tab_size =       glade_xml_get_widget (xml, "tab_width");
  env->undo_actions =   glade_xml_get_widget (xml, "undo_actions");
  env->warning =        glade_xml_get_widget (xml, "warning_color");
  env->wrap =           glade_xml_get_widget (xml, "text_wrapping");
  
  env->label_autoindent =     glade_xml_get_widget (xml, "label_autoindent");
  env->label_bracket =        glade_xml_get_widget (xml, "label_bracketmatch");
  env->label_compiler =       glade_xml_get_widget (xml, "label_compiler");
  env->label_filemanagement = glade_xml_get_widget (xml, "label_filemanagement");
  env->label_font =           glade_xml_get_widget (xml, "label_font");
  env->label_rightmargin =    glade_xml_get_widget (xml, "label_rightmargin");
  env->label_tabstops =       glade_xml_get_widget (xml, "label_tabstops");
  env->label_text_wrap =      glade_xml_get_widget (xml, "label_textwrapping");
  env->label_undo =           glade_xml_get_widget (xml, "label_undo");
  
  // Set the labels to have a bold font
  gtk_widget_modify_font (env->label_autoindent, fd);
  gtk_widget_modify_font (env->label_bracket, fd);
  gtk_widget_modify_font (env->label_compiler, fd);
  gtk_widget_modify_font (env->label_filemanagement, fd);
  gtk_widget_modify_font (env->label_font, fd);
  gtk_widget_modify_font (env->label_tabstops, fd);
  gtk_widget_modify_font (env->label_text_wrap, fd);
  gtk_widget_modify_font (env->label_undo, fd);
  gtk_widget_modify_font (env->label_rightmargin, fd);
  
  return env;
}

GtkWidget* openldev_env_dialog_get_dialog (EnvironmentDialog *env)
{ return env->dialog;
}

int openldev_env_dialog_run (EnvironmentDialog *env)
{
  return gtk_dialog_run (GTK_DIALOG(env->dialog));
}

void openldev_env_dialog_set_autosave (EnvironmentDialog *env, 
                                       int value)
{ gtk_spin_button_set_value (GTK_SPIN_BUTTON(env->autosave), value);
}

void openldev_env_dialog_set_undo (EnvironmentDialog *env, 
                                   int value)
{ gtk_spin_button_set_value (GTK_SPIN_BUTTON(env->undo_actions), value);
}

void openldev_env_dialog_set_tab_size (EnvironmentDialog *env, 
                                       int value)
{ gtk_spin_button_set_value (GTK_SPIN_BUTTON(env->tab_size), value);
}

void openldev_env_dialog_set_autoindent (EnvironmentDialog *env,
                                         bool value)
{ gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(env->autoindent), value);
}

void openldev_env_dialog_set_bracket (EnvironmentDialog *env,
                                      bool value)
{ gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(env->bracket), value);
}

void openldev_env_dialog_set_load_on_launch (EnvironmentDialog *env,
                                             bool value)
{ gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(env->load_on_launch), value);
}

void openldev_env_dialog_set_replace_tabs (EnvironmentDialog *env,
                                           bool value)
{ gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(env->replace_tabs), value);
}

void openldev_env_dialog_set_right_margin (EnvironmentDialog *env,
                                           bool value)
{ gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(env->right_margin), value);
}

void openldev_env_dialog_set_split_words (EnvironmentDialog *env,
                                          bool value)
{ gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(env->split_words), value);
}

void openldev_env_dialog_set_wrap (EnvironmentDialog *env,
                                   bool value)
{ gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(env->wrap), value);
}

void openldev_env_dialog_set_error (EnvironmentDialog *env, 
                                    const char* value)
{
  GdkColor tmp = { 0, 0, 0, 0, },
           *color = gdk_color_copy(&tmp);
  gdk_color_parse (value, color);
  gtk_color_button_set_color (GTK_COLOR_BUTTON(env->error), color);
}

void openldev_env_dialog_set_font (EnvironmentDialog *env, 
                                   const char* face,
                                   const char* size)
{
  env->fontname = string(face) + " " + string(size);
  gtk_font_button_set_font_name (GTK_FONT_BUTTON(env->fontbutton), env->fontname.c_str());
}

void openldev_env_dialog_set_warning (EnvironmentDialog *env, 
                                      const char* value)
{
  GdkColor tmp = { 0, 0, 0, 0, },
           *color = gdk_color_copy(&tmp);
  gdk_color_parse (value, color);
  gtk_color_button_set_color (GTK_COLOR_BUTTON(env->warning), color);
}

int openldev_env_dialog_get_autosave (EnvironmentDialog *env)
{ return gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(env->autosave));
}

int openldev_env_dialog_get_undo (EnvironmentDialog *env)
{ return gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(env->undo_actions));
}

int openldev_env_dialog_get_tab_size (EnvironmentDialog *env)
{ return gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(env->tab_size));
}

bool openldev_env_dialog_get_autoindent (EnvironmentDialog *env)
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(env->autoindent));
}

bool openldev_env_dialog_get_bracket (EnvironmentDialog *env)
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(env->bracket));
}

bool openldev_env_dialog_get_load_on_launch (EnvironmentDialog *env)
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(env->load_on_launch));
}

bool openldev_env_dialog_get_replace_tabs(EnvironmentDialog *env)
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(env->replace_tabs));
}

bool openldev_env_dialog_get_right_margin (EnvironmentDialog *env)
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(env->right_margin));
}

bool openldev_env_dialog_get_split_words (EnvironmentDialog *env)
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(env->split_words));
}

bool openldev_env_dialog_get_wrap (EnvironmentDialog *env)
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(env->wrap));
}

string openldev_env_dialog_get_color_string (GdkColor &color)
{
  char blue[3], green[3], red[3];
  snprintf(blue, sizeof(blue), "%X", color.blue);
  snprintf(red, sizeof(red), "%X", color.red);
  snprintf(green, sizeof(green), "%X", color.green);
  
  string clr = "#" + string(red);
  if (color.red == 0)
    clr += "0";
  clr += green;
  if (color.green == 0)
    clr += "0";
  clr += blue;
  if (color.blue == 0)
    clr += "0";
  
  return clr;
}

const char* openldev_env_dialog_get_error (EnvironmentDialog *env)
{ 
  GdkColor color;
  gtk_color_button_get_color (GTK_COLOR_BUTTON(env->error), &color);

  return openldev_env_dialog_get_color_string(color).c_str(); 
}

const char* openldev_env_dialog_get_font_size (EnvironmentDialog *env)
{
  string font = gtk_font_button_get_font_name(GTK_FONT_BUTTON(env->fontbutton));
  
  while (font.find(" ", 0) != string::npos)
    font.erase(0, font.find(" ", 0)+1);
  
  return font.c_str();
}

const char* openldev_env_dialog_get_font_face (EnvironmentDialog *env)
{ 
  string font = gtk_font_button_get_font_name(GTK_FONT_BUTTON(env->fontbutton));
  
  int loc = font.find(' ', 0);
  while (font.find(' ', loc+1) != string::npos)
    loc = font.find(' ', loc+1);
  
  font.erase(loc, font.length()-loc);
  return font.c_str();
}

const char* openldev_env_dialog_get_warning (EnvironmentDialog *env)
{ 
  GdkColor color;
  gtk_color_button_get_color (GTK_COLOR_BUTTON(env->warning), &color);

  return openldev_env_dialog_get_color_string(color).c_str(); 
}
