/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-new-project.h"

void openldev_new_project_dialog_populate_combobox (NewProjectDialog *project);
void openldev_new_project_dialog_on_apply  (GtkWidget *widget, gpointer data);
void openldev_new_project_dialog_on_browse (GtkWidget *widget, gpointer data);
void openldev_new_project_dialog_on_cancel (GtkWidget *widget, gpointer data);

string openldev_new_project_dialog_check_for_errors (NewProjectDialog*);
string openldev_new_project_dialog_create_odp_file  (NewProjectDialog*, string&);
void openldev_new_project_dialog_create_location    (NewProjectDialog*);
void openldev_new_project_dialog_clean_white_space  (NewProjectDialog*, string&, bool);
void openldev_new_project_dialog_create_project     (NewProjectDialog*, string);

void openldev_new_project_dialog_error      (string message);
void openldev_new_project_dialog_write_file (string filename, string text);

NewProjectDialog* openldev_new_project_dialog_new (TemplateList* list_of_templates)
{
  NewProjectDialog *project = new NewProjectDialog();

  project->tlist = list_of_templates;
  
  string gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/newproject.glade";
  GladeXML *xml = glade_xml_new(gladefile.c_str(), NULL, NULL);

  project->dialog = glade_xml_get_widget(xml, "dialog");
  project->combobox = glade_xml_get_widget(xml, "comboboxentry");
  project->name = glade_xml_get_widget(xml, "name");
  project->version = glade_xml_get_widget(xml, "version");
  project->homepage = glade_xml_get_widget(xml, "homepage");
  project->email = glade_xml_get_widget(xml, "email");
  project->author = glade_xml_get_widget(xml, "author");
  project->location = glade_xml_get_widget(xml, "location");
  project->cancel = glade_xml_get_widget(xml, "button_cancel");
  project->apply = glade_xml_get_widget(xml, "button_ok");
  project->browse = glade_xml_get_widget(xml, "button_browse");
  
  openldev_new_project_dialog_populate_combobox (project);

  string temp = string(g_get_home_dir()) + "/openldev/";
  gtk_entry_set_text(GTK_ENTRY(project->location), temp.c_str());
  temp = "\0";

  g_signal_connect (G_OBJECT(project->apply), "clicked", 
                    G_CALLBACK (openldev_new_project_dialog_on_apply),
                    (gpointer)project);
  g_signal_connect (G_OBJECT(project->cancel), "clicked", 
                    G_CALLBACK(openldev_new_project_dialog_on_cancel), 
                    (gpointer)project);
  g_signal_connect (G_OBJECT(project->browse), "clicked", 
                    G_CALLBACK(openldev_new_project_dialog_on_browse), 
                    (gpointer)project);
  
  return project;
}

const char* openldev_new_project_dialog_run (NewProjectDialog* project)
{
  int result = 0;
  
  while (result == 0)
  { result = gtk_dialog_run (GTK_DIALOG(project->dialog));
  }
  
  switch (result)
  {
    case (GTK_RESPONSE_ACCEPT):
    {
      string text = string(gtk_entry_get_text (GTK_ENTRY(project->location))) 
                  + gtk_entry_get_text (GTK_ENTRY(project->name))
                  + string("/") + gtk_entry_get_text (GTK_ENTRY(project->name))
                  + string(".openldev");
      gtk_widget_destroy (project->dialog);
      return text.c_str();
    }
    default:
    {
      gtk_widget_destroy (project->dialog);
      return "NULL";
    }
  }
  
  return "NULL";
}

// This adds the different types of projects to the combobox
void openldev_new_project_dialog_populate_combobox (NewProjectDialog *project)
{ 
  GtkTreeModel *model;
	GtkTreeIter iter;
  GtkCellRenderer *renderer;

	model = GTK_TREE_MODEL(gtk_list_store_new(1, G_TYPE_STRING));
	
	for (unsigned int i=0; i < project->tlist->title.size(); i++) 
  {
		gtk_list_store_append(GTK_LIST_STORE(model), &iter);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, project->tlist->title.at(i).c_str(), -1);
	}
  
  gtk_combo_box_set_model(GTK_COMBO_BOX(project->combobox), model);
	g_object_unref(model);
  
  renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(project->combobox), renderer, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(project->combobox), renderer, "text", 0, NULL);
  gtk_combo_box_set_active(GTK_COMBO_BOX(project->combobox), 0);
}

// When cancel is pressed, this function is called to emit a cancel signal
// and hide the dialog
void openldev_new_project_dialog_on_cancel (GtkWidget *widget, 
                                            gpointer data)
{
  NewProjectDialog *ptr = (NewProjectDialog*) data;
  gtk_dialog_response(GTK_DIALOG(ptr->dialog), GTK_RESPONSE_CANCEL);
}

// When the browse button is pressed, the user is presented with a 
// FileChooserDialog where they can choose where to create the project.
void openldev_new_project_dialog_on_browse (GtkWidget *widget, 
                                            gpointer data)
{
  NewProjectDialog *ptr = (NewProjectDialog*) data;
  
  GtkWidget *filedialog;
  filedialog = gtk_file_chooser_dialog_new("Choose Directory", NULL,
                GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, 
                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
                GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
  
  int result = gtk_dialog_run(GTK_DIALOG(filedialog));
  switch (result)
  {
    // If OK is pressed
    case (GTK_RESPONSE_ACCEPT):
    {
      string fn = string(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filedialog))) + "/";
      gtk_entry_set_text(GTK_ENTRY(ptr->location), fn.c_str());
      fn = "\0";
      break;
    }
    default:
      break;
  }
  
  gtk_widget_unref (filedialog);
}

// When the apply button is pressed, it is checked that the project name only
// contains letters and numbers and that a version and location is specified.
// The project is then created from a template and the Makefiles and configure
// file are edited. The OpenLDev project file is also created.
void openldev_new_project_dialog_on_apply (GtkWidget *widget, 
                                           gpointer data)
{
  NewProjectDialog *ptr = (NewProjectDialog*) data;  
  string errortext = openldev_new_project_dialog_check_for_errors (ptr);
  
  // Create new project
  if (errortext == "NONE")
  {
    string odp_file_text, odp_file_location, project_path, copy, extract,
           rm, configure_in, makefile_am, makefile_am2, text;
    // Add a '/' to the end of the location if it does not exist
    string locationtext = gtk_entry_get_text(GTK_ENTRY(ptr->location));
    if (locationtext.at(locationtext.length()-1) != '/')
    {
      string str = locationtext;
      str += "/";
      gtk_entry_set_text(GTK_ENTRY(ptr->location), str.c_str());
      str = "\0";
    }

    // Create the project location, openldev file location and openldev file text
    openldev_new_project_dialog_create_location (ptr);
    odp_file_text = openldev_new_project_dialog_create_odp_file (ptr, odp_file_location);
    
    // Create directory path
    project_path += string(gtk_entry_get_text(GTK_ENTRY(ptr->location))) 
                 + gtk_entry_get_text(GTK_ENTRY(ptr->name)) + "/";
    
    // Create copy command, configure.in path, & makefile.am path
    string choice_text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(ptr->combobox));
    
    for (unsigned int i=0; i < ptr->tlist->file.size(); i++)
      if (ptr->tlist->title.at(i) == choice_text)
      {
        openldev_new_project_dialog_create_project (ptr, ptr->tlist->file.at(i));
        i = ptr->tlist->file.size();
      }
    
    // Create the project from the template & create the project file
    openldev_new_project_dialog_write_file (odp_file_location, odp_file_text);
    ptr->file_location = odp_file_location;
    
    odp_file_text = "\0";
    odp_file_location = "\0";
    project_path = "\0";
    copy = "\0";
    extract = "\0";
    rm = "\0";
    configure_in = "\0";
    makefile_am = "\0";
    makefile_am2 = "\0";
    text = "\0";
    
    gtk_dialog_response(GTK_DIALOG(ptr->dialog), GTK_RESPONSE_ACCEPT);
  }
  // Throw an error
  else
    openldev_new_project_dialog_error (errortext);
}

string openldev_new_project_dialog_check_for_errors (NewProjectDialog* project)
{
  string ntxt = gtk_entry_get_text(GTK_ENTRY(project->name)),
         vtxt = gtk_entry_get_text(GTK_ENTRY(project->version)),
         ltxt = gtk_entry_get_text(GTK_ENTRY(project->location));
  
  // Check for error when creating a project or file
  if (ntxt.find(" ",0) != string::npos ||
      ntxt.find("_",0) != string::npos ||
      ntxt.empty())
    return "The project name must\nbe alphanumeric!";
  else if (vtxt.empty())
    return "You must specify a\nversion number!";
  else if (ltxt.empty())
    return "You must specify a\nproject location!";
  else
    return "NONE";
  
  ntxt = "\0";
  vtxt = "\0";
  ltxt = "\0";
}

// Create a new project file. This function returns text that is to be added
// to the project file and returns by reference the location the project file
// is to be created.
string openldev_new_project_dialog_create_odp_file (NewProjectDialog* project,
                                                    string &dir)
{
  string choice_text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(project->combobox)),
         odp_file_text, sources;
  
  if (choice_text.find("C++", 0) != string::npos)
    sources = "main.cc/";
  else
    sources = "main.c/";
  
  // Create the text for the file
  odp_file_text = "<name>" + string(gtk_entry_get_text(GTK_ENTRY(project->name))) 
    + "*\n<author>" + gtk_entry_get_text(GTK_ENTRY(project->author))
    + "*\n<location>" + gtk_entry_get_text(GTK_ENTRY(project->location)) 
    + gtk_entry_get_text(GTK_ENTRY(project->name)) + "/*\n<email>" 
    + gtk_entry_get_text(GTK_ENTRY(project->email)) + "*\n<url>" 
    + gtk_entry_get_text(GTK_ENTRY(project->homepage)) + "*\n<version>" 
    + gtk_entry_get_text(GTK_ENTRY(project->version));
  
  // Add the files that need to be added for the template
  for (unsigned int i=0; i < project->tlist->title.size(); i++)
  {
    if (project->tlist->title.at(i) == choice_text)
    {
      odp_file_text += string("*\n<template>") + project->tlist->file.at(i) 
                    + "*\n<lastfile>" + sources.substr(0, sources.find("/", 0))
                    + "*\n<sources>" + sources
                    + "*\n<headers>*\n<otherfiles>*\n<execname>";
    }
  }
    
  dir = gtk_entry_get_text(GTK_ENTRY(project->location)) 
      + string(gtk_entry_get_text(GTK_ENTRY(project->name))) 
      + "/" + gtk_entry_get_text(GTK_ENTRY(project->name)) + ".openldev";
  
  choice_text = "\0";
  sources = "\0";
  
  return odp_file_text;
}

void openldev_new_project_dialog_create_location (NewProjectDialog* project)
{
  string str = gtk_entry_get_text(GTK_ENTRY(project->location));
  mkdir(str.c_str(), 0777);
  str += gtk_entry_get_text(GTK_ENTRY(project->name)) + string("/");
  mkdir(str.c_str(), 0777);
  str += "src/";
  mkdir(str.c_str(), 0777);
  str = "\0";
}

void openldev_new_project_dialog_clean_white_space (NewProjectDialog* project,
                                                    string &text, 
                                                    bool isMakefile)
{
  string firststring[] = { "AUTHOR", "HOMEPAGE", "EMAIL", "TITLE", "VERSION", "[[", "]]", "AMP" };
  string replacement[] = { gtk_entry_get_text(GTK_ENTRY(project->author)), 
                           gtk_entry_get_text(GTK_ENTRY(project->homepage)), 
                           gtk_entry_get_text(GTK_ENTRY(project->email)), 
                           gtk_entry_get_text(GTK_ENTRY(project->name)), 
                           gtk_entry_get_text(GTK_ENTRY(project->version)), 
                           "<", ">", "&" };
  
  for (unsigned int i=0; i<8; i++)
    while ((text.find(firststring[i], 0) != string::npos))
    {
      if (!(i == 0 && isMakefile))
      {
        int l = text.find(firststring[i], 0);
        text.erase(l, firststring[i].length());
        text.insert(l, replacement[i]);
      }
      else
        break;
    }
  
  int x;
  for (x=0; x < 8; x++)
  {
    firststring[x] = "\0";
    replacement[x] = "\0";
  }
}

void openldev_new_project_dialog_create_project (NewProjectDialog* project,
                                                 string filename)
{
  string file = string(INSTALL_PREFIX) + "/share/openldev/templates/"
                     + filename + ".template";
  xmlDocPtr doc = xmlParseFile (file.c_str());
  xmlNodePtr cur;
  
  if (doc != NULL) 
  {
    cur = xmlDocGetRootElement(doc);
    if (!xmlStrcmp(cur->name, (const xmlChar *) "project"))
    {
      cur = cur->xmlChildrenNode;
      while (cur != NULL)
      {
        if (!xmlStrcmp(cur->name, (const xmlChar *) "basefiles"))
        {
          cur = cur->xmlChildrenNode;
          while (cur != NULL)
          {
            if (!xmlStrcmp(cur->name, (const xmlChar *) "file"))
            {
              string fn = string(gtk_entry_get_text(GTK_ENTRY(project->location)))
                        + gtk_entry_get_text(GTK_ENTRY(project->name)) + "/" 
                        + (const char *)xmlGetProp(cur, (const xmlChar *) "name");
              string content = (const char *)xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
              
              if (content.find("AUTHORS", 0) != string::npos)
                openldev_new_project_dialog_clean_white_space (project, content, true);
              else
                openldev_new_project_dialog_clean_white_space (project, content, false);
              openldev_new_project_dialog_write_file (fn, content);
              
              if (content.find("#!/bin/sh", 0) != string::npos)
                chmod(fn.c_str(), 0777);
              fn = "\0";
              content = "\0";
            }
            
            cur = cur->next;
          }

          break;
        }
        cur = cur->next;
      }
      
      cur = xmlDocGetRootElement(doc);
      cur = cur->xmlChildrenNode;
      while (cur != NULL)
      {
        if (!xmlStrcmp(cur->name, (const xmlChar *) "srcfiles"))
        {
          cur = cur->xmlChildrenNode;
          while (cur != NULL)
          {
            if (!xmlStrcmp(cur->name, (const xmlChar *) "file"))
            {
              string fn = string(gtk_entry_get_text(GTK_ENTRY(project->location))) 
                        + gtk_entry_get_text(GTK_ENTRY(project->name)) + "/src/" 
                        + (const char *)xmlGetProp(cur, (const xmlChar *) "name");
              string content = (const char *) xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
              
              openldev_new_project_dialog_clean_white_space (project, content, false);
              openldev_new_project_dialog_write_file (fn, content);
              if (content.find("#!/bin/sh", 0) != string::npos)
                chmod(fn.c_str(), 0777);
              
              fn = "\0";
              content = "\0";
            }
            
            cur = cur->next;
          }
          
          break;
        }
        cur = cur->next;
      }
    }
  }
  
  xmlFreeDoc(doc);
  file = "\0";
}

// Throw an error dialog with the specified message
void openldev_new_project_dialog_error (string message)
{
  GtkWidget *dialog;
  dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, 
                                   GTK_BUTTONS_CLOSE, message.c_str());
  int result = gtk_dialog_run (GTK_DIALOG(dialog));
  
  switch (result)
  {
    default:
      gtk_widget_hide(dialog);
  }
}
// Write the text to the file. This function receives the filename and the file
// text. The text is then written to the file located at the filename.
void openldev_new_project_dialog_write_file (string filename, 
                                          string text)
{
  ofstream fout(filename.c_str());
  fout << text;
  fout.close();
}
