/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-project-settings.h"

string openldev_project_dialog_read_file (string filename);
void openldev_project_dialog_write_file (string filename, string text);
bool openldev_project_dialog_on_version_change (GtkWidget*, GdkEventFocus*, gpointer);
void openldev_project_dialog_on_files_change (GtkComboBox*, gpointer);
bool openldev_project_dialog_on_filecontent_change (GtkWidget*, GdkEventFocus*, gpointer);

ProjectDialog* openldev_project_dialog_new (ProjectSettings *settings)
{
  ProjectDialog *project = new ProjectDialog();
  
  // Create Makefile.am location string
  string gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/projectproperties.glade";
  GladeXML *xml = glade_xml_new(gladefile.c_str(), NULL, NULL);
  
  project->dialog =  glade_xml_get_widget(xml, "dialog");
  project->version = glade_xml_get_widget(xml, "version");
  project->files = glade_xml_get_widget(xml, "files");
  project->filecontent = glade_xml_get_widget(xml, "filecontent");
  project->settings = settings;
  
  g_signal_connect (G_OBJECT(project->version), 
                    "focus-out-event",
                    G_CALLBACK(openldev_project_dialog_on_version_change),
                    (gpointer)project->settings);
  g_signal_connect (G_OBJECT(project->filecontent), 
                    "focus-out-event",
                    G_CALLBACK(openldev_project_dialog_on_filecontent_change),
                    (gpointer)project);  
  g_signal_connect (G_OBJECT(project->files),
                    "changed",
                    G_CALLBACK(openldev_project_dialog_on_files_change),
                    (gpointer)project);

  // Create entries for project properties
  gtk_entry_set_text (GTK_ENTRY(project->version), settings->version.c_str());
  gtk_widget_modify_font (project->filecontent, 
                          pango_font_description_from_string("Monospace 9"));
  gtk_combo_box_set_active (GTK_COMBO_BOX(project->files), 0);
  
  return project;
}

int openldev_project_dialog_run (ProjectDialog* project)
{
  int result = gtk_dialog_run (GTK_DIALOG(project->dialog));
  
  switch (result)
  {
    default:
      gtk_widget_hide (project->dialog);
      return result;
  }
}

bool openldev_project_dialog_on_version_change (GtkWidget *version,
                                                GdkEventFocus *event,
                                                gpointer data)
{
  ProjectSettings *settings = (ProjectSettings*)data;
  openldev_project_settings_set_version(settings, gtk_entry_get_text(GTK_ENTRY(version)));
  
  return false;
}

bool openldev_project_dialog_on_filecontent_change (GtkWidget *filecontent,
                                                    GdkEventFocus *event,
                                                    gpointer data)
{
  ProjectDialog *project = (ProjectDialog*)data;
  
  string file = project->settings->location 
              + gtk_combo_box_get_active_text (GTK_COMBO_BOX(project->files));
  GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(filecontent));
  GtkTextIter start, end;
  
  gtk_text_buffer_get_start_iter (buffer, &start);
  gtk_text_buffer_get_end_iter (buffer, &end);
  openldev_project_dialog_write_file (file, gtk_text_buffer_get_text(buffer, &start, &end, false));
  file = "\0";
  
  return false;
}


void openldev_project_dialog_on_files_change (GtkComboBox *combobox,
                                              gpointer data)
{
  ProjectDialog *project = (ProjectDialog*)data;

  string file = project->settings->location + gtk_combo_box_get_active_text (combobox);
  file = openldev_project_dialog_read_file (file);
  
  gtk_text_buffer_set_text (gtk_text_view_get_buffer(GTK_TEXT_VIEW(project->filecontent)), 
                            file.c_str(), file.length());
}

// Read in the text from a file. This function receives a filename and returns
// the text string of the file.
string openldev_project_dialog_read_file (string filename)
{
  if (!g_file_test(filename.c_str(), G_FILE_TEST_EXISTS))
    return string("");
  
  string text;
  string line;
  ifstream fin(filename.c_str());
  while (getline(fin, line))
    text += g_locale_to_utf8 (line.c_str(), -1, NULL, NULL, NULL) + string("\n");
  line = "\0";
  filename = "\0";
  
  return text;
}

// text. The text is then written to the file located at the filename.
void openldev_project_dialog_write_file (string filename, 
                                         string text)
{
  ofstream fout(filename.c_str());
  fout << text;
  fout.close();
}

const char* openldev_project_dialog_get_version (ProjectDialog* project)
{ return gtk_entry_get_text (GTK_ENTRY(project->version));
}
