/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-file.h"

void openldev_on_status_bar_update (GtkTextBuffer*, GtkTextIter*, GtkTextMark*, gpointer);
void openldev_on_delete_tab (GtkWidget*, gpointer);

void openldev_file_new_file (GtkMenuItem *item, 
                             gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev->project->name.empty())
    openldev_error ("Please open or create\na project first!");
  // Only create new file if project is open
  else 
  {
    const char *file;
    NewFileDialog *dialog = openldev_new_file_dialog_new (openldev->project->location.c_str());
    openldev_new_file_dialog_run (dialog, file);

    string fn = file;
    if (fn != "NULL" && fn != "")
    {
      openldev_browser_project_refresh (openldev->browser->files);
      openldev_open_file (openldev, fn.c_str());     
    }
    
    fn = "\0";
    file = "\0";
    gtk_widget_destroy (dialog->dialog);
    delete dialog;
  }
}

// Create a new project by calling the new project dialog & then open the
// project that was created.
void openldev_file_new_project (GtkMenuItem* menuitem, 
                                gpointer data)
{  
  OpenLDev *openldev = (OpenLDev*) data;
  
  NewProjectDialog *dialog = openldev_new_project_dialog_new (openldev->templatelist);
  string project_name = openldev_new_project_dialog_run (dialog);

  // Open newly created project if OK is pressed
  if (project_name != "NULL")
  {
    openldev_open_project (openldev, project_name.c_str());
    openldev_menu_build_auto_generate (openldev);
  }
  
  project_name = "\0";
  delete dialog;
}

void openldev_file_open (GtkMenuItem *item, 
                         gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;

  // Create file selector
  GtkWidget *dialog;
  dialog  = gtk_file_chooser_dialog_new("Choose a project or File", 
                                        GTK_WINDOW(openldev->window),
                                        GTK_FILE_CHOOSER_ACTION_OPEN, 
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                        GTK_STOCK_OPEN, GTK_RESPONSE_OK,
                                        NULL);
  
  // Set default folder
  if (!openldev->project->name.empty())
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog),
                                         openldev->project->location.c_str());
  else
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog),
                                         g_get_home_dir());  
  
  int result = gtk_dialog_run (GTK_DIALOG(dialog));
  switch (result)
  {
    // If OK is pressed
    case (GTK_RESPONSE_OK):
    {
      // Check if file is a .openldev file
      string filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(dialog));
      gtk_widget_destroy (dialog);
      
      if (filename.find(".openldev", 0) != string::npos)
        openldev_open_project (openldev, filename.c_str());
      else
        openldev_open_file (openldev, filename.c_str());
      filename = "\0";
      break;
    }
    default:
      gtk_widget_destroy (dialog);
      break;
  }
}

// Open the specified file if it exists. This function receives the filename that
// it is supposed to open. If a project is open, the file is added as the
// last opened file. The file is then opened, syntax highlighting is applied, 
// line numbering is calculated and displayed, signals are connected to the
// keystroke action and final initialization is applied
void openldev_open_file (OpenLDev *openldev,
                         const char* file)
{
  string filename = file;
  
  if (!g_file_test (filename.c_str(), G_FILE_TEST_EXISTS))
    openldev_error ("Cannot open file. It does not exist!");
  else
  {
    // Add a new tab to the notebookmanager
    if (openldev_file_manager_add_tab (openldev->notebook,
                                       filename.c_str(),
                                       openldev->env))
    { 
      string text, title, cliptext;

      // Set the file as a last opened file
      if (!openldev->project->name.empty())
      {
        string temp = "(" + filename + ")";
        if (openldev->project->lastfiles.find(temp, 0) == string::npos)
          openldev_project_settings_set_lastfiles (openldev->project,
                           (openldev->project->lastfiles + temp).c_str());
        temp = "\0";
      }
  
      // Open file & read in text
      text = openldev_read_file (openldev_file_manager_get_current_filename(openldev->notebook));
      
      // Add text to text buffer after applying syntax highlighting
      GtkSourceBuffer *buffer = openldev_file_manager_get_textbuffer(openldev->notebook);
      gtk_source_buffer_begin_not_undoable_action (buffer);
      gtk_text_buffer_set_text (GTK_TEXT_BUFFER(buffer), text.c_str(), -1);
      gtk_source_buffer_end_not_undoable_action (buffer);
      
      // Set the text to the buffer, connect keystroke function & show/hide line numbering
      gtk_text_view_set_buffer (GTK_TEXT_VIEW(openldev_file_manager_get_textview(openldev->notebook)),
                                GTK_TEXT_BUFFER(buffer));
      
      g_signal_connect (G_OBJECT(openldev_file_manager_get_textbuffer(openldev->notebook)), 
                        "mark-set",
                        G_CALLBACK(openldev_on_status_bar_update), 
                        (gpointer)openldev);
      g_signal_connect (G_OBJECT(openldev_file_manager_get_textview(openldev->notebook)), 
                        "key-press-event",
                        G_CALLBACK(openldev_on_textview_keypress), 
                        (gpointer)openldev);
      g_signal_connect (G_OBJECT(openldev_file_manager_get_textview(openldev->notebook)), 
                        "event",
                        G_CALLBACK(openldev_on_textview_event), 
                        (gpointer)openldev);
      g_signal_connect (G_OBJECT(openldev->notebook->tabs[openldev->notebook->tab_count-1]->button),
                        "clicked",
                        G_CALLBACK(openldev_on_delete_tab),
                        (gpointer)openldev);
      openldev_file_manager_set_properties (openldev->notebook, 
                                            false, openldev->env);
      
      string font_description = openldev->env->font_face + " " + openldev->env->font_size;
      PangoFontDescription *fd = pango_font_description_from_string(font_description.c_str());
      openldev_file_manager_modify_fonts (openldev->notebook, fd);
      
      text = "\0";
      title = "\0";
      cliptext = "\0";
      font_description = "\0";
    }
  }
}

// The project specified is opened. This function receives the filename of the
// project file. If the project was loaded, the last file opened is reloaded,
// the filelist is populated and the last project open is set to this project
void openldev_open_project (OpenLDev *openldev,
                            const char *file)
{
  string filename = file;
  openldev_file_manager_erase_all_tabs (openldev->notebook);
  bool success = openldev_project_settings_load_project_file (openldev->project, filename.c_str());
 
  // If the project was opened
  if (success)
  {
    // Open the last file if it exists
    delete openldev->ptemplate;
    openldev->ptemplate = openldev_project_template_new (openldev->project->template_name.c_str());
    
    string files = openldev->project->lastfiles;
    while (!files.empty())
    {
      int l = files.find(")", 0);
      string temp = files.substr(0, l+1);
      files.erase (0, l+1);
      
      temp.erase (0, 1);
      temp.erase (temp.length()-1, 1);
      openldev_open_file (openldev, temp.c_str());
      temp = "\0";
    }

    // Make environment ready for project
    openldev_populate_treeview (openldev);
    openldev_environment_set_last_project (openldev->env, filename.c_str());

    vector<string> fns = openldev_project_settings_get_sources_all (openldev->project);
    openldev_file_manager_autolist_parse_files (openldev->notebook, fns);
    
    fns.clear();
    filename = "\0";
    files = "\0";
    
    gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                              "/MenuBar/File/FileClose"), true);
    gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                              "/MenuBar/Project"), true);
    gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                              "/MenuBar/Build"), true);
    gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                              "/ToolBar/BuildCompile"), true);
    gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                              "/ToolBar/BuildBuild2"), true);
    gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                              "/ToolBar/BuildExecute"), true);
    gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                              "/ToolBar/BuildClean"), true);
    gtk_widget_set_sensitive (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                              "/ToolBar/BuildStop"), true);
  }
}

// Save the current file & mark the file as unmodified. If the file does not
// have a name, then save as is called
void openldev_file_save (GtkMenuItem* menuitem, 
                         gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev_file_manager_get_file_open (openldev->notebook))
  {
    string current_fn = openldev_file_manager_get_current_filename (openldev->notebook);
    if (current_fn.empty() || current_fn == "EMPTY")
      openldev_menu_file_save_as (openldev);
    current_fn = "\0";
  
    // Save the file & set as unmodified
    GtkTextIter start, end;
    GtkSourceBuffer *buffer = openldev_file_manager_get_textbuffer (openldev->notebook);
    
    gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER(buffer), &start);
    gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER(buffer), &end);
    openldev_write_file (openldev_file_manager_get_current_filename (openldev->notebook),
                         gtk_text_buffer_get_text (GTK_TEXT_BUFFER(buffer), &start, &end, true));

    openldev_file_manager_mark_current_tab_modified (openldev->notebook, false);
    string file = openldev_file_manager_get_current_filename (openldev->notebook),
           loc = openldev->project->location + "src/";

    if (openldev_project_settings_get_file_in_project (openldev->project, file.c_str()))
    {
      file = openldev_file_manager_get_current_short_filename (openldev->notebook);
      openldev_file_manager_autolist_parse_file (openldev->notebook, file.c_str(), loc.c_str());
    }
    
    file = "\0";
    loc = file;
  }
}

// Save the current file by a different name. The FileChooserDialog allows the
// user to choose the location and filename. Note: This function does not actually
// save the file, but rather just chooses a new name for it & calls on_file_save
void openldev_file_save_as (GtkMenuItem* menuitem, 
                            gpointer data) 
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev_file_manager_get_file_open (openldev->notebook))
  {
    GtkWidget *dialog;
    dialog = gtk_file_chooser_dialog_new ("Save file as", 
                                          GTK_WINDOW(openldev->window),
                                          GTK_FILE_CHOOSER_ACTION_SAVE,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          GTK_STOCK_SAVE, GTK_RESPONSE_OK,
                                          NULL);
  
    // Set default folder
    string folder_name = openldev->project->location + "src/";
    
    if (!openldev->project->name.empty())
      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog), 
                                           folder_name.c_str());  
    else
      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog), 
                                           g_get_home_dir());
    folder_name = "\0";
  
    // Add the filename to the text entry in SaveAs dialog
    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(dialog),
          openldev_file_manager_get_current_short_filename(openldev->notebook).c_str());
  
    // Open fileselection dialog to get file location
    int result = gtk_dialog_run (GTK_DIALOG(dialog));
    switch (result)
    {
      // If OK is pressed
      case (GTK_RESPONSE_OK):
      {
        openldev_file_manager_set_filename (openldev->notebook,
                 gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog)));
        openldev_menu_file_save (openldev);
        break;
      }
      default:
        break;
    }
    
    gtk_widget_destroy (dialog);
  }
}

// Call the notebookmanager function to save all of the files that are
// currently open
void openldev_file_save_all (GtkMenuItem* menuitem, 
                             gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev_file_manager_get_file_open (openldev->notebook))
    openldev_file_manager_save_all_files (openldev->notebook);
}

// If the user chooses to allow it, revert to the saved version of a file. Then
// the syntax highlighting, action manager and line numbering are refreshed
void openldev_file_revert (GtkMenuItem* menuitem, 
                           gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev_file_manager_get_file_open (openldev->notebook))
    if (openldev_question("Are you sure you want to reload this file?\nAny unsaved changes will be lost."))
      openldev_file_manager_revert_to_saved (openldev->notebook);
}

// Update the status bar. This function receives the iterator of the cursor
// position and the mark at that point. It then calculates the line and column
// position and adds them with the project name an version to the status bar.
// The project and version are labelled as 'N/A' if a project is not open.
void openldev_on_status_bar_update (GtkTextBuffer *buffer,
                                    GtkTextIter *insert, 
                                    GtkTextMark *mark,
                                    gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  string message = "\0";
  char vertical[5], horizontal[5];
  g_snprintf(vertical, sizeof(vertical), "%d", gtk_text_iter_get_line(insert)+1);
  g_snprintf(horizontal, sizeof(horizontal), "%d", gtk_text_iter_get_line_offset(insert)+1);
  
  if (!openldev->project->name.empty())
  {
    message += "Project: " + openldev->project->name + "     ";
    message += "Version: " + openldev->project->version + "     ";
  }
  
  message += "Line: " + string(vertical) + "     ";
  message += "Column: " + string(horizontal);
  
  openldev_status_bar_set_message (openldev->statusbar, message.c_str());
  message = "\0";
}

void openldev_on_delete_tab (GtkWidget *widget,
                             gpointer data)
{
  OpenLDev *openldev = (OpenLDev*)data;
  string temp, file;

  for (int i=0; i < openldev->notebook->tab_count; i++)
    if (openldev->notebook->tabs[i]->button == widget)
    {
      file = "(" + openldev->notebook->tabs[i]->fn + ")";
      if (openldev->project->lastfiles.find(file, 0) != string::npos)
      {
        temp = openldev->project->lastfiles;
        temp.erase (temp.find(file, 0), file.length());
        openldev_project_settings_set_lastfiles (openldev->project, temp.c_str());
        temp = "\0";
      }
      file = "\0";
    }
  
  openldev_file_manager_on_delete_tab (widget, (gpointer)openldev->notebook);
}
