/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-printing.h"

void openldev_file_print (GtkMenuItem *menuitem,
                          gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev_file_manager_get_file_open(openldev->notebook))
  {  
    string file = string("File: ") + openldev_file_manager_get_current_short_filename(openldev->notebook),
           font = openldev->env->font_face + " Regular " + openldev->env->font_size;
    GnomePrintConfig *cfg = gnome_print_config_default();
    GtkSourcePrintJob *sjob = gtk_source_print_job_new (cfg); 
  
    gtk_source_print_job_set_buffer (sjob, GTK_SOURCE_BUFFER(openldev_file_manager_get_textbuffer(openldev->notebook)));
    gtk_source_print_job_set_tabs_width (sjob, openldev->env->tab_size);
    gtk_source_print_job_set_wrap_mode (sjob, GTK_WRAP_NONE);
    gtk_source_print_job_set_highlight (sjob, false);
    gtk_source_print_job_set_print_numbers (sjob, 5);
    gtk_source_print_job_set_text_margins (sjob, 0.5, 0.5, 0.5, 0.5);
    gtk_source_print_job_set_font (sjob, font.c_str());
    gtk_source_print_job_set_numbers_font (sjob, font.c_str());
  
    GnomePrintJob *job = gtk_source_print_job_print (sjob);
    g_object_unref (G_OBJECT(sjob));
    GtkWidget *dialog = gnome_print_dialog_new (job, (guchar*)"Print Document",
                        GNOME_PRINT_DIALOG_RANGE | GNOME_PRINT_DIALOG_COPIES);
  
    gint result = gtk_dialog_run (GTK_DIALOG(dialog));
    gtk_widget_destroy (dialog);
  
    if (result == GNOME_PRINT_DIALOG_RESPONSE_PREVIEW)
      openldev_menu_file_print_preview (openldev);
    else if (result == GNOME_PRINT_DIALOG_RESPONSE_PRINT)
      gnome_print_job_print (job);
  
    g_object_unref (G_OBJECT(job));
    g_object_unref (G_OBJECT(cfg));
    file = "\0";
    font = "\0";
  }
}

void openldev_file_print_preview (GtkMenuItem *menuitem,
                                  gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (openldev_file_manager_get_file_open(openldev->notebook))
  {  
    string file = string("File: ") + openldev_file_manager_get_current_short_filename(openldev->notebook),
           font = openldev->env->font_face + " Regular " + openldev->env->font_size;
    GnomePrintConfig *cfg = gnome_print_config_default();
    GtkSourcePrintJob *sjob = gtk_source_print_job_new (cfg); 

    gtk_source_print_job_set_buffer (sjob, GTK_SOURCE_BUFFER(openldev_file_manager_get_textbuffer(openldev->notebook)));
    gtk_source_print_job_set_tabs_width (sjob, openldev->env->tab_size);
    gtk_source_print_job_set_wrap_mode (sjob, GTK_WRAP_NONE);
    gtk_source_print_job_set_highlight (sjob, false);
    gtk_source_print_job_set_print_numbers (sjob, 5);
    gtk_source_print_job_set_text_margins (sjob, 0.5, 0.5, 0.5, 0.5);
    gtk_source_print_job_set_font (sjob, font.c_str());
    gtk_source_print_job_set_numbers_font (sjob, font.c_str());
  
    GnomePrintJob *job = gtk_source_print_job_print (sjob);
    g_object_unref (G_OBJECT(sjob));
    GtkWidget *preview;

    preview = gnome_print_job_preview_new (job, (guchar*)"Print Preview");
    gtk_widget_show(preview);
  
    g_object_unref (job);
    g_object_unref (cfg);
    file = "\0";
    font = "\0";
  }
}
