/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-auto-list.h"

void openldev_auto_list_add_file    (AutoList*, string);
void openldev_auto_list_add_word    (AutoList*, string, int);
void openldev_auto_list_fix_word_id (int&);
void openldev_auto_list_parse_file  (AutoList*, const char*);
bool openldev_auto_list_check_char  (char);
int  openldev_auto_list_find_word   (AutoList*, string);

AutoList* openldev_auto_list_new ()
{
  AutoList *list = new AutoList();
  return list;
}

vector<string> openldev_auto_list_get_matches (AutoList *list, 
                                               string match)
{
  int id = match[0];
  openldev_auto_list_fix_word_id (id);
  int len = match.length(), location = -1;
  vector<string> choices;

  for (unsigned int i = 0; i < list->words[id].word_list.size(); i++)
    if (list->words[id].word_list.at(i).word.substr(0, len) == match)
    {
      location = i;
      break;
    }
  
  if (location != -1)
    for (int w = location; w < location + 3; w++)
      if (w < (signed int)list->words[id].word_list.size())
        if (list->words[id].word_list.at(w).word.substr(0, len) == match &&
            list->words[id].word_list.at(w).word != match)
          choices.push_back(list->words[id].word_list.at(w).word);
    
  return choices;
}

void openldev_auto_list_add_word (AutoList *list, 
                                  string word)
{
  int fword = openldev_auto_list_find_word (list, word),
      id = word[0];
  
  openldev_auto_list_fix_word_id (id);
  
  if (fword != -1)
    return;
  else
  {
    AutoListWord wd;
    wd.word = word;
    
    if (list->words[id].word_list.size() > 0)
    {
      unsigned int count = 0;
      vector<AutoListWord>::iterator iter = list->words[id].word_list.begin();
    
      while (list->words[id].word_list.at(count).word < wd.word)
      {
        count++;
        iter++;
        
        if (list->words[id].word_list.size() == count)
          break;
      }

      if (list->words[id].word_list.size() == count)
        list->words[id].word_list.push_back (wd);
      else
        list->words[id].word_list.insert (iter, wd);
    }
    else
      list->words[id].word_list.push_back (wd);
  }
}

bool openldev_auto_list_check_char (char ch)
{
  if (isalnum(ch) || ch == '_' || ch == '-')
    return true;
  else
    return false;
}

void openldev_auto_list_clean_all (AutoList *list)
{  
  for (int i=0; i < 27; i++)
    list->words[i].word_list.clear();
}

void openldev_auto_list_fix_word_id (int &wordid)
{
  if (wordid == 95)
    wordid = 26;
  else if (wordid > 96)
    wordid -= 97;
  else
    wordid -= 65;
}

void openldev_auto_list_parse_file (AutoList *list, const char *file)
{
  char buffer[256];
  string text, word;
  ifstream fin(file);
  
  while (!fin.eof())
  {
    fin.getline (buffer, 255);
    text = string(buffer);

    while (text.length() > 0)
    {
      if (openldev_auto_list_check_char(text[0]))
        word += text[0];
      else if (!word.empty() && word.length() >= 4 &&
               (isalpha(word[0]) || word[0] == '_'))
      {
        openldev_auto_list_add_word (list, word);
        word.clear();
      }
      else
        word.clear();

      text.erase(0, 1);
    }
    
    if (!word.empty() && word.length() >= 4 &&
        (isalpha(word[0]) || word[0] == '_'))
    {
      openldev_auto_list_add_word (list, word);
      word.clear();
    }
  }
  
  fin.close();
}

void openldev_auto_list_parse_files (AutoList *list, 
                                     vector<string> &fns)
{
  for (unsigned int i=0; i < fns.size(); i++)
    openldev_auto_list_parse_file (list, fns.at(i).c_str());
}

int openldev_auto_list_find_word (AutoList* list, 
                                  string word)
{
  int id = word[0];
  openldev_auto_list_fix_word_id (id);
  int high = list->words[id].word_list.size(), low = -1, mid;
  
  if (high > 0)
  {
    while ((high - low) > 1)
    {
      mid = (high + low) / 2;
      if (list->words[id].word_list.at(mid).word > word)
        high = mid;
      else
        low = mid;
    }
    
    if (low == -1 || list->words[id].word_list.at(low).word != word)
      return -1;
    else
      return low;
  }
  
  return -1;
}
