/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-makefile-parser.h"

bool openldev_makefile_parser_is_blank    (string);
void openldev_makefile_parser_add_array   (Makefile*, string&);
void openldev_makefile_parser_add_attr    (Makefile*, string);
void openldev_makefile_parser_add_comment (Makefile*, string);
void openldev_makefile_parser_add_script  (Makefile*, string);
void openldev_makefile_parser_add_string  (Makefile*, string);

Makefile* openldev_makefile_parser_new (const char *file)
{
  Makefile *mf = new Makefile;
  mf->filename = file;
  string line;
  bool usecurrent = false;
  
  // Loop for every line in the Makefile.am
  ifstream fin (file);
  while (getline (fin, line))
  {
    // If usecurrent==true, that means that there is a script or
    //  array that may have another line
    if (usecurrent)
    {
      if (mf->types[mf->types.size()-1] == MAKEFILE_SCRIPT)
      {
        // Add the script if the line begins with a tab
        if (line[0] == 9)
        {
          line.erase (0, 1);
          mf->attributes[mf->attributes.size()-1]->attr.push_back(line);
        }
        // Otherwise, fall through
        else
          usecurrent = false;
      }
      else if (mf->types[mf->types.size()-1] == MAKEFILE_ARRAY)
      {
        openldev_makefile_parser_add_attr (mf, line);
        
        // Stop any parsing if this is the last line of the array
        if (line.find ("\\", 0) == string::npos)
        {
          usecurrent = false;
          line = "DO NOT PARSE";
        }
      }
      // Just a catch for odd cases
      else
        usecurrent = false;
    }

    if (!usecurrent && line != "DO NOT PARSE")
    {
      if (line[0] == '#')
        openldev_makefile_parser_add_comment (mf, line);
      else if (line.find("=", 0) != string::npos)
      {
        int L = line.find("\\", 0);
        if (L != (signed)string::npos)
        {
          if (L != (signed)line.length()-1)
          {
            if ((line[L+1] == ' ') || (line[L+1] == 9))
              usecurrent = true;
          }
          else
            usecurrent = true;
        }
        
        openldev_makefile_parser_add_array (mf, line);
        openldev_makefile_parser_add_attr (mf, line);
      }
      else if (line.find(":", 0) != string::npos)
      {
        openldev_makefile_parser_add_script (mf, line);
        usecurrent = true;
      }
      else if (!openldev_makefile_parser_is_blank(line))
        openldev_makefile_parser_add_string (mf, line);
    }
  }
  
  fin.close();
  return mf;
}

void openldev_makefile_parser_write (Makefile *mf)
{
  string content;
  unsigned int i, j;
  
  for (i=0; i < mf->types.size(); i++)
  {
    switch (mf->types[i])
    {
      case (MAKEFILE_STRING):
      {
        content += mf->names[i] + "\n";
        break;
      }
      case (MAKEFILE_COMMENT):
      {
        content += mf->names[i] + "\n";
        break;
      }
      case (MAKEFILE_SCRIPT):
      {
        content += mf->names[i] + ":\n";
        for (j=0; j < mf->attributes[i]->attr.size(); j++)
          content += "\t" + mf->attributes[i]->attr[j] + "\n";
        break;
      }
      case (MAKEFILE_ARRAY):
      {
        if (mf->attributes[i]->attr.size() > 0)
        {
          content += mf->names[i] + " = \\\n";
          for (j=0; j < mf->attributes[i]->attr.size()-1; j++)
            content += "\t" + mf->attributes[i]->attr[j] + "\\\n";
          content += "\t" + mf->attributes[i]->attr[mf->attributes[i]->attr.size()-1] + "\n";
        }
        else
          content += mf->names[i] + " = \n";
        break;
      }
      default:
        content += "Error: Makefile string type not recognized!\n";
    }
    
    content += "\n";
  }
  
  ofstream fout(mf->filename.c_str());
  fout << content;
  fout.close();
  content = "\0";
}

vector<string> openldev_makefile_parser_get_subdirs (Makefile *mf)
{
  for (unsigned int i=0; i < mf->types.size(); i++)
    if ((mf->types[i] == MAKEFILE_ARRAY) &&
        (mf->names[i] == "SUBDIRS"))
      return mf->attributes[i]->attr;
  
  vector<string> empty;
  return empty;
}

vector<string> openldev_makefile_parser_get_sources (Makefile *mf)
{
  vector<string> sources;
  for (unsigned int i=0; i < mf->types.size(); i++)
    if ((mf->types[i] == MAKEFILE_ARRAY) &&
        (mf->names[i].find("SOURCES", 0) != string::npos))
      for (unsigned int j=0; j < mf->attributes[i]->attr.size(); j++)
        sources.push_back (mf->attributes[i]->attr[j]);
  
  return sources;
}

bool openldev_makefile_parser_is_blank (string line)
{
  for (unsigned int i=0; i < line.length(); i++)
    if ((line[i] != ' ') && (line[i] != 9))
      return false;
  
  return true;
}

void openldev_makefile_parser_add_array (Makefile *mf,
                                         string &line)
{
  string name = line;
  name = line.substr (0, line.find("=", 0));
  line.erase (0, line.find("=", 0)+1);
  while (name.find(" ", 0) != string::npos)
    name.erase (name.find(" ", 0), 1); 

  MakefileAttr *obj = new MakefileAttr;
  mf->types.push_back (MAKEFILE_ARRAY);
  mf->attributes.push_back (obj);
  mf->names.push_back (name);
  name = "\0";
}

void openldev_makefile_parser_add_attr (Makefile *mf,
                                        string line)
{
  if (line == "DO NOT PARSE")
    return;

  // Clean up all slashes and front whitespace
  int L = line.find ("\\", 0);
  while (L != (signed)string::npos)
  {
    if (L != (signed)line.length()-1)
    {
      if ((line[L+1] == ' ') || (line[L+1] == 9))
        line.erase (L, 1);
    }
    else
      line.erase (L, 1);
    
    L = line.find ("\\", L+1);
  }
  while ((line[0] == ' ') || (line[0] == 9))
    line.erase (0, 1);
  for (unsigned int i=0; i < line.length(); i++)
    if (line[i] == 9)
      line[i] = ' ';
  
  while (!line.empty())
  {
    if (line.find(" ", 0) != string::npos)
    {
      string attr = line.substr (0, line.find(" ", 0));
      if (!attr.empty())
        mf->attributes[mf->attributes.size()-1]->attr.push_back (attr);
      line.erase (0, line.find(" ", 0) + 1);
    }
    else
    {
      mf->attributes[mf->attributes.size()-1]->attr.push_back (line);
      line = "\0";
    }
  }
}

void openldev_makefile_parser_add_comment (Makefile *mf,
                                           string line)
{
  MakefileAttr *obj = new MakefileAttr;
  mf->types.push_back (MAKEFILE_COMMENT);
  mf->attributes.push_back (obj);
  mf->names.push_back (line);
}

void openldev_makefile_parser_add_script (Makefile *mf,
                                          string line)
{
  string name = line;
  name = line.substr (0, line.find(":", 0));
  line.erase (0, line.find(":", 0)+1);
  while (name.find(" ", 0) != string::npos)
    name.erase (name.find(" ", 0), 1);
  while ((line[0] == ' ') || (line[0] == 9))
    line.erase (0, 1);

  MakefileAttr *obj = new MakefileAttr;
  if (!line.empty())
    obj->attr.push_back (line);
  mf->types.push_back (MAKEFILE_SCRIPT);
  mf->attributes.push_back (obj);
  mf->names.push_back (name);
  name = "\0";
}

void openldev_makefile_parser_add_string (Makefile *mf,
                                          string line)
{
  MakefileAttr *obj = new MakefileAttr;
  mf->types.push_back (MAKEFILE_STRING);
  mf->attributes.push_back (obj);
  mf->names.push_back (line);
}
