/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-project.h"

void openldev_project_settings_write_project_file (ProjectSettings*);
void openldev_project_settings_get_makefiles (vector<string>&, string);
vector<string> openldev_project_settings_get_subdirs (string);
vector<string> openldev_project_settings_get_makefile_sources (string);

string openldev_project_settings_get_attrib   (string, string&);
string openldev_project_settings_read_file    (string);
void   openldev_project_settings_write_file   (string, string);
string openldev_project_settings_load_version (string);
void openldev_project_settings_error (const char*);

ProjectSettings* openldev_project_settings_new (const char* filename)
{
  ProjectSettings* settings = new ProjectSettings();
  
  if (filename == "")
    return settings;
  else if (openldev_project_settings_load_project_file(settings, filename))
    return settings;
  else
    return NULL;
}

void openldev_project_settings_close_project (ProjectSettings* settings)
{
  openldev_project_settings_write_project_file (settings);
  settings->lastfiles = "";
  settings->location = "";
  settings->name = "";
  settings->template_name = "";
  settings->version = "";
}

bool openldev_project_settings_load_project_file (ProjectSettings* settings, 
                                                  const char* file)
{
  string txt, text, filename = file;

  if (!g_file_test(filename.c_str(), G_FILE_TEST_EXISTS))
    return false;
    
  // Open project file
  text = openldev_project_settings_read_file (filename);
  
  settings->location = "/";
  filename.erase (0, 1);
  while (filename.find("/", 0) != string::npos)
  {
    settings->location += filename.substr (0, filename.find("/", 0) + 1);
    filename.erase (0, filename.find("/", 0) + 1);
  }
  
  // Get all of the attributes
  settings->name = filename.substr (0, filename.find(".openldev", 0));
  settings->lastfiles = openldev_project_settings_get_attrib("<lastfiles>", text);
  settings->template_name = openldev_project_settings_get_attrib("<template>", text);
  settings->version = openldev_project_settings_load_version (settings->location + "configure.in");
  
  // Create the template from the language for backward compatibility
  if (settings->template_name == "")
    settings->template_name = "console-c";
  
  while (settings->lastfiles.find(" ", 0) != string::npos)
    settings->lastfiles.erase (settings->lastfiles.find(" ", 0), 1);
  
  txt = "\0";
  text = "\0";
  filename = "\0";

  return true; 
}

bool openldev_project_settings_get_file_in_project (ProjectSettings *settings, 
                                                    const char *filename)
{
  string file = filename, location;
  while (file.find("/", 0) != string::npos)
  {
    location += file.substr (0, file.find("/", 0) + 1);
    file.erase (0, file.find("/", 0) + 1);
  }

  vector<string> sources = openldev_project_settings_get_makefile_sources (location + "Makefile.am");
  for (unsigned int i = 0; i < sources.size(); i++)
    if (sources[i] == file)
    {
      sources.clear();
      return true;
    }
  
  sources.clear();
  return false;
}

void openldev_project_settings_write_project_file (ProjectSettings* settings)
{
  string odp, fn;
    
  // Create .ODP file text
  odp = "<template>" + settings->template_name + "*\n<lastfiles>" + settings->lastfiles + "*\n";
  fn += settings->location + settings->name + ".openldev";
  
  // Write the file
  openldev_project_settings_write_file (fn, odp);
  fn = "\0";
  odp = "\0";
}

// Read in the text from a file. This function receives a filename and returns
// the text string of the file.
string openldev_project_settings_read_file (string filename)
{
  if (!g_file_test(filename.c_str(), G_FILE_TEST_EXISTS))
    return string("");
  
  string text, line;
  ifstream fin(filename.c_str());
  while (getline(fin, line))
    text += g_locale_to_utf8 (line.c_str(), -1, NULL, NULL, NULL) + string("\n");
  
  return text;
}

// Write the text to the file. This function receives the filename and the file
// text. The text is then written to the file located at the filename.
void openldev_project_settings_write_file (string filename, 
                                           string text)
{
  ofstream fout(filename.c_str());
  fout << text;
  fout.close();
}

// Get specific attribute from the text. The function receives the attribute
// and the text of the project file. It returns that text of the attribute.
string openldev_project_settings_get_attrib (string attribute, 
                                             string &text)
{
  int start, len;
  
  if (text.find(attribute, 0) != string::npos)
  {
    start = text.find(attribute, 0) + attribute.length();
    len = text.find("*", start) - start;
    return text.substr(start, len);
  }
  
  return " "; 
}

string openldev_project_settings_load_version (string file)
{
  string text = openldev_project_settings_read_file (file);
  int l1, l2 = text.find ("AM_INIT_AUTOMAKE", 0);
  
  l1 = text.find (",", l2) + 1;
  l2 = text.find (")", l1);
  
  text = text.substr (l1, l2-l1);
  while (text.find(" ", 0) != string::npos)
    text.erase (text.find(" ", 0), 1);
  
  return text; 
}

void openldev_project_settings_set_lastfiles (ProjectSettings* settings,
                                              const char* value)
{
  settings->lastfiles = value;
  openldev_project_settings_write_project_file (settings);
}

void openldev_project_settings_set_version (ProjectSettings* settings,
                                            const char* value)
{
  settings->version = value;
  
  string file = settings->location + "configure.in";
  string text = openldev_project_settings_read_file (file);
  int l1, l2 = text.find ("AM_INIT_AUTOMAKE", 0);
  
  l1 = text.find (",", l2) + 1;
  l2 = text.find (")", l1);
  
  text.erase (l1, l2 - l1);
  text.insert (l1, " ");
  text.insert (l1 + 1, value);
  
  openldev_project_settings_write_file (file.c_str(), text.c_str());
  file = "\0";
  text = "\0";
}

vector<string> openldev_project_settings_get_makefile_list (ProjectSettings *project)
{
  vector<string> makefiles;
  openldev_project_settings_get_makefiles (makefiles, project->location);
  
  return makefiles;
}

void openldev_project_settings_get_makefiles (vector<string> &makefiles,
                                              string base_dir)
{
  string makefile = base_dir + "Makefile.am";
  vector<string> subdirs = openldev_project_settings_get_subdirs (makefile);

  for (unsigned int i=0; i < subdirs.size(); i++)
    openldev_project_settings_get_makefiles (makefiles,
      (base_dir + subdirs.at(i) + "/"));

  makefiles.push_back (makefile);
  subdirs.clear();
  makefile = "\0";
}

vector<string> openldev_project_settings_get_subdirs (string makefile)
{
  Makefile *mf = openldev_makefile_parser_new (makefile.c_str());
  vector<string> subdirs = openldev_makefile_parser_get_subdirs (mf);
  delete mf;

  return subdirs;
}

vector<string> openldev_project_settings_get_sources (ProjectSettings *project, 
                                                      const char *base_dir, 
                                                      bool subdirs)
{
  vector<string> makefiles, sources;
  
  if (subdirs)
    openldev_project_settings_get_makefiles (makefiles, base_dir);
  else
    makefiles.push_back(string(base_dir) + "Makefile.am");
  
  for (unsigned int i=0; i < makefiles.size(); i++)
  {
    string makefile = makefiles[i].substr(0, makefiles[i].find("Makefile.am"));
    vector<string> temp = openldev_project_settings_get_makefile_sources (makefiles[i]);
    for (unsigned int j=0; j < temp.size(); j++)
    {        
      if (subdirs)
        sources.push_back(makefile + temp[j]);
      else
        sources.push_back(temp[j]);
    }
    temp.clear();
  }
  
  makefiles.clear();
  return sources;
}

vector<string> openldev_project_settings_get_sources_all (ProjectSettings *project)
{ return openldev_project_settings_get_sources (project, project->location.c_str(), true);
}

vector<string> openldev_project_settings_get_makefile_sources (string file)
{
  Makefile *mf = openldev_makefile_parser_new (file.c_str());
  vector<string> files = openldev_makefile_parser_get_sources (mf);
  delete mf;
  
  return files;
}

// Throw an error dialog with the specified message
void openldev_project_settings_error (const char *message)
{
  GtkWidget *dialog;
  dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, 
                                   GTK_BUTTONS_CLOSE, message);
  int result = gtk_dialog_run (GTK_DIALOG(dialog));
  
  switch (result)
  {
    default:
      gtk_widget_destroy(dialog);
  }
}
