/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-tools.h"

void openldev_search_all_files (vector<string>&, vector<string>&, string, string);
void openldev_mark_found_text (OpenLDev*, GtkTextIter*, GtkTextIter*);

// Toggle the visibility of the goto line toolbar. Also, this makes sure that if
// the goto line toolbar is visible, the find toolbar is hidden
void openldev_edit_goto_line (GtkMenuItem *menuitem,
                              gpointer data) 
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (GTK_WIDGET_VISIBLE(openldev->gototoolbar->hbox))
    gtk_widget_hide (openldev->gototoolbar->hbox);
  else
  {
    gtk_widget_hide (openldev->findtoolbar->hbox);
    gtk_widget_show (openldev->gototoolbar->hbox);
    gtk_widget_grab_focus (openldev->gototoolbar->goto_line);
  }
}

// Move to the specified line number in the buffer. If the line exists, the
// cursor is moved to the beginning of that line and the notebookmanager scrolls
// to it. If the line does not exist (e.g. It is out of range), then the goto
// line error message and image are shown, not to be hidden until a successful
// goto line is completed.
void openldev_goto_to_line_number (OpenLDev *openldev, int line)
{
  if (openldev_file_manager_get_file_open (openldev->notebook))
  {
    GtkTextBuffer *buffer = GTK_TEXT_BUFFER(openldev_file_manager_get_textbuffer(openldev->notebook));
  
    // If the line is within the bounds of the document
    if (gtk_text_buffer_get_line_count(buffer) >= openldev_goto_toolbar_get_goto_value(openldev->gototoolbar)
        || line >= 0)
    {
      GtkTextIter iter;
      if (line < 0)
        gtk_text_buffer_get_iter_at_line(buffer, &iter, 
          openldev_goto_toolbar_get_goto_value(openldev->gototoolbar) - 1);
      else
        gtk_text_buffer_get_iter_at_line(buffer, &iter, line);
      gtk_text_buffer_move_mark_by_name(buffer, "insert", &iter);
      gtk_text_buffer_move_mark_by_name(buffer, "selection_bound", &iter);
      openldev_file_manager_scroll_to_mark (openldev->notebook);
      gtk_widget_grab_focus(openldev_file_manager_get_textview(openldev->notebook));
    
      openldev_goto_toolbar_show_error (openldev->gototoolbar, false);
    }
    // If the line is out of bounds...
    else
      openldev_goto_toolbar_show_error (openldev->gototoolbar, true);
  }
}

// Toggle the visibility of the find toolbar. Also, this makes sure that if
// the Find toolbar is visible, the goto line toolbar is hidden
void openldev_edit_find (GtkMenuItem *menuitem,
                         gpointer data) 
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (GTK_WIDGET_VISIBLE(openldev->findtoolbar->hbox))
    gtk_widget_hide (openldev->findtoolbar->hbox);
  else
  {
    gtk_widget_hide (openldev->gototoolbar->hbox);
    gtk_widget_show (openldev->findtoolbar->hbox);
    gtk_widget_grab_focus (openldev->findtoolbar->find_text);
  }
}

void openldev_edit_find_next (GtkMenuItem *menuitem,
                              gpointer data) 
{ openldev_find_next ((OpenLDev*) data);
}

// Find the next iteration of the text the user specified in the buffer.
// If there is no selected text, search from the current cursor position.
// Otherwise, search from the selection bound. If the text is not found, wrap
// around the document once and then throw an error if it is still not found.
void openldev_find_next (OpenLDev *openldev)
{
  // Check if there was no text specified
  string entry_text = gtk_entry_get_text (GTK_ENTRY(openldev->findtoolbar->find_text));
  if (!entry_text.empty())
  {
    if (openldev_find_toolbar_search_files_is_active(openldev->findtoolbar))
    {
      gtk_notebook_set_current_page(GTK_NOTEBOOK(openldev->output->notebook), 1);
      if (gtk_tree_view_get_model(GTK_TREE_VIEW(openldev->output->search->search_box)))
        gtk_tree_view_remove_column(GTK_TREE_VIEW(openldev->output->search->search_box),
            gtk_tree_view_get_column(GTK_TREE_VIEW(openldev->output->search->search_box), 0));                            
      
      vector<string> search_results;
      vector<string> sources = openldev_project_settings_get_sources_all (openldev->project);
      openldev_search_all_files (search_results, sources, openldev->project->location, entry_text);
      
      if (search_results.size() > 0)
      {
        GtkTreeModel *model;
	      GtkTreeIter iter;
        GtkCellRenderer *cell_renderer;
        GtkTreeViewColumn *column;
        
        model = GTK_TREE_MODEL(gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING));
        
        cell_renderer = gtk_cell_renderer_text_new();
	      column = gtk_tree_view_column_new_with_attributes("Search Results", 
                          cell_renderer, "text", 0, "foreground", 1, NULL);
        gtk_tree_view_append_column(GTK_TREE_VIEW(openldev->output->search->search_box), column);
	
	      for (unsigned int i=0; i < search_results.size(); i++) 
        {
		      gtk_list_store_append(GTK_LIST_STORE(model), &iter);
		      gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, search_results[i].c_str(), 1, "Red", -1);
	      }
    
        gtk_tree_view_set_model(GTK_TREE_VIEW(openldev->output->search->search_box), model);
	      g_object_unref(model);
        
        openldev_find_toolbar_show_error (openldev->findtoolbar, false);
      }
      else
        openldev_find_toolbar_show_error (openldev->findtoolbar, true);
      
      search_results.clear();
      sources.clear();
    }
    else
    {
      GtkSourceBuffer *buffer = openldev_file_manager_get_textbuffer (openldev->notebook);
      GtkTextIter iter, end, enditer;
      gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER(buffer), &enditer);
        
      // If there is no selected text, then search from the cursor position
      gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER(buffer), &iter, 
        gtk_text_buffer_get_selection_bound (GTK_TEXT_BUFFER(buffer)));
      gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER(buffer), &end, 
        gtk_text_buffer_get_insert (GTK_TEXT_BUFFER(buffer)));
      if (gtk_text_iter_compare (&end, &iter) > 0)
        iter = end;
  
      // If it is found, select the text
      if (gtk_text_iter_forward_search (&iter, 
          gtk_entry_get_text (GTK_ENTRY(openldev->findtoolbar->find_text)), 
          GTK_TEXT_SEARCH_VISIBLE_ONLY, &iter, &end, &enditer))
            openldev_mark_found_text (openldev, &iter, &end);
      // Throw an error if it cannot be found
      else 
      {
        gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(buffer), &iter);
        if (gtk_text_iter_forward_search(&iter, entry_text.c_str(), 
            GTK_TEXT_SEARCH_VISIBLE_ONLY, &iter, &end, &enditer))
              openldev_mark_found_text (openldev, &iter, &end);
        else
          openldev_find_toolbar_show_error (openldev->findtoolbar, true);
      }
    }
  }
  
  entry_text = "\0";
}

// Search all of the files in a project for the text you want to find. This
// function receives the vector of search results to edit by reference, and
// strings with the source and header files, the location of the project
// directory and the text to search for in the files.
void openldev_search_all_files (vector<string> &search_results, 
                                vector<string> &sources,
                                string location,
                                string find_text)
{
  string text;
  gchar buff[5];

  // while there are files left to search
  while (sources.size() > 0)
  {
    int line = 1;
  
    // Open the file & get it line by line
    ifstream fin(sources[0].c_str());
    sources[0].erase (0, location.length());
    while (getline(fin, text))
    {
      // If the text is found, add a string to the search results vector in
      // the form - filename:line: Line Content
      if (text.find(find_text, 0) != string::npos)
      {
        snprintf(buff, sizeof(buff), "%d", line);
        search_results.push_back(string(sources[0] + ":" + buff + ": " + text));
      }
      line++;
    }
    
    fin.close();
    sources.erase(sources.begin());
  }
  
  text = "\0";
}

// Mark the found text by selecting it. This function receives the beginning
// and end iterators of the found text. When this function is called, the error
// message and image on the find toolbar is hidden
void openldev_mark_found_text (OpenLDev *openldev,
                               GtkTextIter *iter, 
                               GtkTextIter *end)
{
  GtkTextBuffer *buffer = GTK_TEXT_BUFFER(openldev_file_manager_get_textbuffer(openldev->notebook));
  gtk_text_buffer_move_mark(buffer, gtk_text_buffer_get_insert(buffer), iter);
  gtk_text_buffer_move_mark(buffer, gtk_text_buffer_get_selection_bound(buffer), end);
  openldev_file_manager_scroll_to_mark (openldev->notebook);
  gtk_widget_grab_focus(openldev_file_manager_get_textview(openldev->notebook));
      
  openldev_find_toolbar_show_error (openldev->findtoolbar, false);
}

// Replace the specified text with the text specified by the user in the input
// dialog. By choosing the checkbox to search in all files, you can replace the
// text in all of the files.
void openldev_find_and_replace (OpenLDev *openldev)
{
  int count = 0;
  string replace_text;
  openldev_find_next (openldev);
  
  // Loop until the text is no longer found
  while (openldev_file_manager_get_insert_offset(openldev->notebook) != 
         openldev_file_manager_get_selection_offset(openldev->notebook))
  {
    // Get the text to replace with from the user
    replace_text = openldev_input_dialog_run ("Replace With: ", "Replace Text");
    
    // Cancel the replacing
    if (replace_text == "EMPTY")
      break;
    
    // Erase the found text & replace it
    GtkTextBuffer *buffer = GTK_TEXT_BUFFER(openldev_file_manager_get_textbuffer(openldev->notebook));
    GtkTextIter insert, select;
    
    gtk_text_buffer_get_iter_at_mark(buffer, &insert,
             gtk_text_buffer_get_insert(buffer));
    gtk_text_buffer_get_iter_at_mark(buffer, &select,
             gtk_text_buffer_get_selection_bound(buffer));
    gtk_text_buffer_delete(buffer, &insert, &select);
    gtk_text_buffer_insert_at_cursor(buffer, replace_text.c_str(), -1);
    
    // Refresh the cursor and find the next iteration
    gtk_text_buffer_get_iter_at_mark(buffer, &select,
             gtk_text_buffer_get_selection_bound(buffer));
    gtk_text_buffer_move_mark_by_name(buffer, "insert", &select);
    
    openldev_find_next (openldev);
    count++;
  }
  
  // Hide the error if there was text found
  if (count > 0)
    openldev_find_toolbar_show_error (openldev->findtoolbar, false);
  
  replace_text = "\0";
}
