/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-file-manager.h"

PangoTabArray* openldev_file_manager_get_tab_array (PangoFontDescription*, EnvironmentSettings*, GtkWidget*);
string openldev_file_manager_read_file (string filename);

void openldev_file_manager_set_properties_one_tab (FileManager*, int, EnvironmentSettings*);
void openldev_file_manager_on_notebook_event      (GtkNotebook*, gint, gpointer);
bool openldev_file_manager_on_textview_keypress   (GtkWidget*, GdkEvent*, gpointer);
bool openldev_file_manager_on_textview_focus_out  (GtkWidget*, GdkEventFocus*, gpointer);
void openldev_file_manager_write_file             (string filename, string text);

FileManager* openldev_file_manager_new ()
{
  FileManager *manager = new FileManager();
 
  manager->autolist = openldev_auto_list_new();
  manager->notebook = gtk_notebook_new();
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK(manager->notebook), true);
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK(manager->notebook), GTK_POS_TOP);
  gtk_notebook_set_scrollable (GTK_NOTEBOOK(manager->notebook), true);
  gtk_widget_show (manager->notebook);
  
  manager->acompl = new AutoComplete();
  manager->acompl->autocomplete = gtk_tree_view_new ();
  manager->acompl->autocompletewin = gtk_window_new (GTK_WINDOW_POPUP);
  gtk_container_add (GTK_CONTAINER(manager->acompl->autocompletewin), manager->acompl->autocomplete);
  gtk_container_set_border_width (GTK_CONTAINER(manager->acompl->autocompletewin), 2);
  gtk_window_set_decorated (GTK_WINDOW(manager->acompl->autocompletewin), false);
  gtk_window_set_skip_taskbar_hint (GTK_WINDOW(manager->acompl->autocompletewin), false);
  gtk_window_set_gravity (GTK_WINDOW(manager->acompl->autocompletewin), GDK_GRAVITY_NORTH_WEST);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(manager->acompl->autocomplete), false);
  
  g_signal_connect (G_OBJECT(manager->notebook), "change_current_page",
                    G_CALLBACK(openldev_file_manager_on_notebook_event), (gpointer)manager);
  manager->tab_count = 0;
  
  return manager;
}

GtkWidget* openldev_file_manager_get_textview (FileManager *manager)
{ 
  int current = gtk_notebook_get_current_page(GTK_NOTEBOOK(manager->notebook));

  // Return the current textview
  if (manager->tab_count != 0)
    return manager->tabs[current]->textview;
  else
    return NULL;
}

GtkSourceBuffer* openldev_file_manager_get_textbuffer (FileManager *manager)
{ return GTK_SOURCE_BUFFER(gtk_text_view_get_buffer(GTK_TEXT_VIEW(openldev_file_manager_get_textview(manager))));
}

string openldev_file_manager_get_current_filename (FileManager *manager)
{ return manager->tabs[gtk_notebook_get_current_page(GTK_NOTEBOOK(manager->notebook))]->fn;
}

string openldev_file_manager_get_current_short_filename (FileManager *manager)
{
  int current = gtk_notebook_get_current_page(GTK_NOTEBOOK(manager->notebook));
  string fn = gtk_label_get_text(GTK_LABEL(manager->tabs[current]->label));
  if (fn.find("*", 0) != string::npos)
    fn.erase(fn.find("*", 0), 1);
  
  return fn;
}

int openldev_file_manager_get_current_tab (FileManager *manager)
{ return gtk_notebook_get_current_page(GTK_NOTEBOOK(manager->notebook));
}

int openldev_file_manager_get_insert_offset (FileManager *manager)
{
  GtkTextBuffer *buffer = GTK_TEXT_BUFFER(openldev_file_manager_get_textbuffer(manager));
  GtkTextIter iter;
  
  gtk_text_buffer_get_iter_at_mark(buffer, &iter,
      gtk_text_buffer_get_insert(buffer));
  return gtk_text_iter_get_offset(&iter);
}

string get_mime_type (string fn)
{ return gnome_vfs_get_mime_type(fn.c_str());
}

int openldev_file_manager_get_selection_offset (FileManager *manager)
{ 
  GtkTextBuffer *buffer = GTK_TEXT_BUFFER(openldev_file_manager_get_textbuffer(manager));
  GtkTextIter iter;
  
  gtk_text_buffer_get_iter_at_mark(buffer, &iter,
      gtk_text_buffer_get_selection_bound(buffer));
  return gtk_text_iter_get_offset(&iter);
}

bool openldev_file_manager_get_file_open (FileManager *manager)
{
  if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(manager->notebook)) > 0)
    return true;
  
  return false;
}
  
void openldev_file_manager_set_filename (FileManager *manager,
                                         const char* fn)
{
  string file = fn;
  while (file.find('/',0) != string::npos)
    file.erase(0, file.find('/',0) + 1);
  
  int current = gtk_notebook_get_current_page(GTK_NOTEBOOK(manager->notebook));
  gtk_label_set_text(GTK_LABEL(manager->tabs[current]->label), file.c_str());
  file = "\0";
}

void openldev_file_manager_set_properties (FileManager* manager, 
                                           bool all_files, 
                                           EnvironmentSettings *env)
{
  if (all_files)
    for (int i=0; i < manager->tab_count; i++)
      openldev_file_manager_set_properties_one_tab (manager, i, env);
  else
    openldev_file_manager_set_properties_one_tab(manager, 
      gtk_notebook_get_current_page(GTK_NOTEBOOK(manager->notebook)), env);
}

void openldev_file_manager_set_properties_one_tab (FileManager *manager,
                                                   int tab,
                                                   EnvironmentSettings *env)
{
  GtkSourceView *view = GTK_SOURCE_VIEW(manager->tabs[tab]->textview);
  GtkSourceBuffer *buffer = GTK_SOURCE_BUFFER(gtk_text_view_get_buffer(GTK_TEXT_VIEW(manager->tabs[tab]->textview)));
  
  gtk_source_view_set_show_line_numbers(view, env->line_numbers);
  gtk_source_view_set_tabs_width(view, env->tab_size);
  gtk_source_view_set_auto_indent(view, env->auto_indent);
  gtk_source_view_set_show_margin(view, env->right_margin);
  gtk_source_view_set_insert_spaces_instead_of_tabs(view, env->replace_tabs);
  gtk_source_buffer_set_max_undo_levels(buffer, env->undo);
  gtk_source_buffer_set_check_brackets(buffer, env->brackets);

  string mime_type = get_mime_type(manager->tabs[tab]->fn); 
  if (mime_type == "NONE")
    gtk_source_buffer_set_highlight(buffer, false);
  else
  {
    gtk_source_buffer_set_highlight(buffer, true);
    GtkSourceLanguagesManager *manager = gtk_source_languages_manager_new();
    GtkSourceLanguage* language = gtk_source_languages_manager_get_language_from_mime_type(manager, mime_type.c_str());

    if (language)
      gtk_source_buffer_set_language(buffer, language);
  }
  mime_type = "\0";
  
  if (env->wrap)
  {
    if (env->wrap_split)
      gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_CHAR);
    else
      gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_WORD);
  }
  else
      gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_NONE);
}

bool openldev_file_manager_add_tab (FileManager *manager,
                                    const char* file,
                                    EnvironmentSettings *env)
{
  string filename = file;
  FileTab *tab = openldev_file_tab_new (filename.c_str());
  tab->fn = filename;
  
  // Parse the filename to remove directory structure
  while (filename.find('/',0) != string::npos)
    filename.erase(0,filename.find('/',0)+1);

  // Check to make sure file is not already open
  for (int i=0; i < manager->tab_count; i++)
  {    
    if (tab->fn == manager->tabs[i]->fn)
    {
      gtk_notebook_set_current_page(GTK_NOTEBOOK(manager->notebook), i);
      return false;
    }
  }

  // Create temporary tab & initialize the objects
	tab->tab_id = manager->tab_count;

  // Add the tab to the notebook and notebook_tab's deque
	manager->tabs.push_back(tab);
  gtk_notebook_append_page(GTK_NOTEBOOK(manager->notebook),
                           manager->tabs[manager->tab_count]->swin,
                           manager->tabs[manager->tab_count]->hbox);

  // Set the font and tab size of the textview  
  string font_description = env->font_face + " " + env->font_size;
  PangoFontDescription *fd = pango_font_description_from_string (font_description.c_str());
  gtk_text_view_set_tabs (GTK_TEXT_VIEW(manager->tabs[manager->tab_count]->textview), 
      openldev_file_manager_get_tab_array(fd, env, manager->tabs[manager->tab_count]->textview));
  font_description = "\0";
  
  // Do final initialization
  gtk_notebook_set_current_page (GTK_NOTEBOOK(manager->notebook), manager->tab_count);
	manager->tab_count++;
  return true;
}

void openldev_file_manager_erase_all_tabs (FileManager *manager)
{
  // Remove all of the tabs and tab information
  manager->tabs.erase(manager->tabs.begin(), manager->tabs.end());
  while (gtk_notebook_get_n_pages(GTK_NOTEBOOK(manager->notebook)) != 0)
    gtk_notebook_remove_page(GTK_NOTEBOOK(manager->notebook), 0);
  
  manager->tab_count = 0;
}

void openldev_file_manager_save_all_files (FileManager *manager)
{
  GtkTextIter start, end;
  
  for (int i=0; i < gtk_notebook_get_n_pages(GTK_NOTEBOOK(manager->notebook)); i++)
  {    
    // Set the label as unmodified
    string text = gtk_label_get_text(GTK_LABEL(manager->tabs[i]->label));
    if (text.find('*', 0) != string::npos)
    {
      text.erase(text.find('*', 0), 1);
      gtk_label_set_text(GTK_LABEL(manager->tabs[i]->label), text.c_str());
    
      // Save the file if it is unmodified
      GtkTextBuffer *buffer = GTK_TEXT_BUFFER (gtk_text_view_get_buffer(GTK_TEXT_VIEW(manager->tabs[i]->textview)));
      gtk_text_buffer_get_start_iter (buffer, &start);
      gtk_text_buffer_get_end_iter (buffer, &end);
      openldev_file_manager_write_file (manager->tabs[i]->fn, gtk_text_buffer_get_text (buffer, &start, &end, true));
    }
    text = "\0";
  }
}

void openldev_file_manager_mark_current_tab_modified (FileManager *manager,
                                                      bool modified)
{
  int current = gtk_notebook_get_current_page(GTK_NOTEBOOK(manager->notebook));
  string text = gtk_label_get_text(GTK_LABEL(manager->tabs[current]->label));
  
  // If the file was modified
  if (modified)
  {
    if (text.find('*', 0) == string::npos)
      text += "*";
  }
  // If it was not modified
  else 
  {
    if (text.find('*', 0) != string::npos)
      text.erase(text.find('*', 0), 1);
  }
  
  gtk_label_set_text(GTK_LABEL(manager->tabs[current]->label), text.c_str());
  text = "\0";
}
  
void openldev_file_manager_scroll_to_mark (FileManager *manager)
{
  GtkTextBuffer *buffer = GTK_TEXT_BUFFER(openldev_file_manager_get_textbuffer(manager));
  GtkTextIter iter;
  gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
  gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(openldev_file_manager_get_textview(manager)), &iter, 0.0, false, 0.0, 0.0);
}

void openldev_file_manager_modify_fonts (FileManager *manager,
                                         PangoFontDescription *fd)
{
  // Foreach tab, modify the fonts
  for (int i=0; i < manager->tab_count; i++)
    gtk_widget_modify_font(manager->tabs[i]->textview, fd);
  gtk_widget_modify_font(manager->acompl->autocomplete, fd);
}

void openldev_file_manager_revert_to_saved (FileManager *manager)
{
  openldev_file_manager_mark_current_tab_modified(manager, false);
  
  int current = gtk_notebook_get_current_page (GTK_NOTEBOOK(manager->notebook));
  string text = openldev_file_manager_read_file (manager->tabs[current]->fn);
  
  GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(manager->tabs[current]->textview));
  gtk_text_buffer_set_text(buffer, text.c_str(), -1);
  gtk_text_view_set_buffer(GTK_TEXT_VIEW(manager->tabs[current]->textview), 
                           GTK_TEXT_BUFFER(buffer));
  text = "\0";
}
 
void openldev_file_manager_autolist_parse_files (FileManager *manager, 
                                                 vector<string>& files)
{
  openldev_file_manager_autolist_clean_all (manager);
  openldev_auto_list_parse_files (manager->autolist, files);
}

void openldev_file_manager_autolist_parse_file (FileManager *manager,
                                                const char *file,
                                                const char *location)
{
  vector<string> files;
  string fn = string(location) + file;
  files.push_back (fn);
  openldev_auto_list_parse_files (manager->autolist, files);
  files.clear();
}

void openldev_file_manager_autolist_clean_all (FileManager *manager)
{ openldev_auto_list_clean_all (manager->autolist);
}

void openldev_file_manager_on_notebook_event (GtkNotebook *widget,
                                              gint page,
                                              gpointer data)
{
  FileManager *ptr = (FileManager*) data;
  gint current_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(widget));
  gtk_widget_grab_focus(ptr->tabs[current_page]->textview);
}

void openldev_file_manager_on_delete_tab (GtkWidget *widget,
                                          gpointer data)
{
  FileManager *ptr = (FileManager*) data;
  
  for (int i=0; i < ptr->tab_count; i++)
    if (ptr->tabs[i]->button == widget)
    {
      string label_text = gtk_label_get_text(GTK_LABEL(ptr->tabs[i]->label));
     
      if (label_text.find('*', 0) != string::npos)
        if (!openldev_file_manager_question("The file you are closing is not saved.\nAre you sure you want to close?"))
          return;

      vector<FileTab*>::iterator itr = ptr->tabs.begin();
      gtk_notebook_remove_page(GTK_NOTEBOOK(ptr->notebook), i);

      for (int count=0; count < i; count++)
        itr++;

      ptr->tab_count--;
      ptr->tabs.erase(itr);

	    if (ptr->tab_count != 0)
        gtk_notebook_set_current_page(GTK_NOTEBOOK(ptr->notebook), i-1);
      label_text = "\0";
      break;
    }    
}

// Set the tab width for when the file contains tabs instead of spaces. This
// function receives the font description, environment settings object and the
// current textview. It returns the tabarray from the textview
PangoTabArray* openldev_file_manager_get_tab_array (PangoFontDescription *fd, 
                                                    EnvironmentSettings *env,
                                                    GtkWidget *text_view)
{
  PangoTabArray *tab_array;
  PangoLayout *layout;
	string tab;
	int i=0, width, height;

  // Add spaces for the number of spaces per tab
	while (i < env->tab_size) 
  {
		tab += " ";
		i++;
	}

  // Create the pango layout & get the pixel width
  layout = gtk_widget_create_pango_layout(text_view, tab.c_str());
  pango_layout_set_font_description(layout, fd);
  pango_layout_get_pixel_size(layout, &width, &height);
 
  // Create the tab array from the width
  tab_array = pango_tab_array_new(1, true);
  pango_tab_array_set_tab(tab_array, 0, PANGO_TAB_LEFT, width);

  return tab_array;
}

// Write the text to the file. This function receives the filename and the file
// text. The text is then written to the file located at the filename.
void openldev_file_manager_write_file (string filename, 
                                       string text)
{
  ofstream fout(filename.c_str());
  fout << text;
  fout.close();
}

// Read in the text from a file. This function receives a filename and returns
// the text string of the file.
string openldev_file_manager_read_file (string filename)
{
  if (!g_file_test(filename.c_str(), G_FILE_TEST_EXISTS))
    return string("");
  
  string text;
  string line;
  ifstream fin(filename.c_str());
  while (getline(fin, line))
    text += g_locale_to_utf8 (line.c_str(), -1, NULL, NULL, NULL) + string("\n");
  
  return text;
}

// Show a question dialog with the specified message. Return the answer the
// user choses. This function handles creating the dialog, setting the properties
// and then deleting it and returning the answer.
bool openldev_file_manager_question (string question)
{
  GtkWidget *dialog = gtk_dialog_new(),
            *yes = gtk_button_new_from_stock("gtk-yes"),
            *no = gtk_button_new_from_stock("gtk-no"),
            *image = gtk_image_new_from_stock("gtk-dialog-warning", GTK_ICON_SIZE_DIALOG),
            *label = gtk_label_new(question.c_str()),
            *hbox = gtk_hbox_new(false, 10);
  
  gtk_container_set_border_width (GTK_CONTAINER(hbox), 10);
  gtk_box_pack_start (GTK_BOX(hbox), image, false, false, 0);
  gtk_box_pack_start (GTK_BOX(hbox), label, false, false, 0);
  
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox),
                      hbox, TRUE, TRUE, 0);
  gtk_box_set_spacing (GTK_BOX(GTK_DIALOG(dialog)->vbox), 7);
  gtk_window_set_title (GTK_WINDOW(dialog), "Question");  
  gtk_window_set_modal (GTK_WINDOW(dialog), true);
  gtk_window_set_resizable (GTK_WINDOW(dialog), false);
  gtk_dialog_set_has_separator (GTK_DIALOG(dialog), true);
  gtk_dialog_add_action_widget (GTK_DIALOG(dialog), no, -9);
  gtk_dialog_add_action_widget (GTK_DIALOG(dialog), yes, -8);
  
  gtk_widget_show_all(dialog);
  
  switch (gtk_dialog_run(GTK_DIALOG(dialog)))
  {
    case -8:
    {
      gtk_widget_destroy (dialog);
      return true;
      break;
    }
    default:
    {
      gtk_widget_destroy (dialog);
      return false;
      break;
    }
  }
}
