/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef FILE_MANAGER_H
#define FILE_MANAGER_H

#include "openldev-file-tab.h"
#include "../openldev-settings/openldev-auto-list.h"
#include "../openldev-settings/openldev-environment.h"
#include "openldev-auto-complete.h"

#include <libgnomevfs/gnome-vfs.h>
#include <gtksourceview/gtksourceview.h>
#include <gtksourceview/gtksourcebuffer.h>
#include <gtksourceview/gtksourcelanguagesmanager.h>
#include <vector>
using namespace std;

typedef struct _FileManager  FileManager;

struct _FileManager
{
  GtkWidget    *notebook;
  AutoComplete *acompl;
  AutoList     *autolist;
  
	vector<FileTab*> tabs;
	int tab_count;
};

FileManager* openldev_file_manager_new ();

GtkWidget* openldev_file_manager_get_textview           (FileManager*);
GtkSourceBuffer* openldev_file_manager_get_textbuffer   (FileManager*);
string openldev_file_manager_get_current_filename       (FileManager*);
string openldev_file_manager_get_current_short_filename (FileManager*);
int    openldev_file_manager_get_current_tab            (FileManager*);
int    openldev_file_manager_get_insert_offset          (FileManager*);
int    openldev_file_manager_get_selection_offset       (FileManager*);
bool   openldev_file_manager_get_file_open              (FileManager*);
  
void openldev_file_manager_set_filename   (FileManager*, const char*);
void openldev_file_manager_set_properties (FileManager*, bool, EnvironmentSettings*);
bool openldev_file_manager_add_tab        (FileManager*, const char*, EnvironmentSettings*);
void openldev_file_manager_erase_all_tabs (FileManager*);
void openldev_file_manager_save_all_files (FileManager*);
void openldev_file_manager_mark_current_tab_modified (FileManager*, bool);
  
void openldev_file_manager_scroll_to_mark  (FileManager*);
void openldev_file_manager_modify_fonts    (FileManager*, PangoFontDescription*);
void openldev_file_manager_revert_to_saved (FileManager*);
 
void openldev_file_manager_autolist_parse_files (FileManager*, vector<string>&);
void openldev_file_manager_autolist_parse_file  (FileManager*, const char*, const char*);
void openldev_file_manager_autolist_clean_all   (FileManager*);

bool openldev_file_manager_question (string question);

// Don't use me!
void openldev_file_manager_on_delete_tab (GtkWidget*, gpointer);

#endif
