/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-file-tab.h"

FileTab* openldev_file_tab_new (const char* filename)
{
  FileTab *tab = new FileTab();
  
  string fn = filename;
  int location = fn.find('/', 0);
  while (location != (signed int)string::npos) 
  {
    fn.erase(0,location+1);
    location = fn.find('/', 0);
  }
  
  tab->fn = filename;
  tab->textview = gtk_source_view_new();
  tab->swin = gtk_scrolled_window_new(NULL, NULL);
  tab->label = gtk_label_new (fn.c_str());
  tab->image = gtk_image_new_from_stock("gtk-close", GTK_ICON_SIZE_MENU);
  tab->hbox = gtk_hbox_new(false, 0);
  tab->button = gtk_button_new();
  
  // Create the textview & add it to the scrolled window
  gtk_text_view_set_editable (GTK_TEXT_VIEW(tab->textview), true);
  gtk_text_view_set_left_margin (GTK_TEXT_VIEW(tab->textview), 2);
  GTK_WIDGET_SET_FLAGS (tab->textview, GTK_CAN_FOCUS);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(tab->swin), GTK_SHADOW_IN);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(tab->swin),
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);
  gtk_container_add (GTK_CONTAINER(tab->swin), tab->textview);

  // Create the button & add the close image to it
  gtk_widget_set_size_request (tab->button, 18, 18);
  gtk_button_set_relief (GTK_BUTTON(tab->button), GTK_RELIEF_NONE);
  gtk_container_add (GTK_CONTAINER(tab->button), tab->image);
	
  // Add the label and button the the tab hbox
  gtk_box_pack_start (GTK_BOX(tab->hbox), tab->label, false, false, 6);
  gtk_box_pack_start (GTK_BOX(tab->hbox), tab->button, false, false, 0);
  
  // Show all of the objects
  gtk_widget_show(tab->textview);
  gtk_widget_show(tab->swin);
  gtk_widget_show(tab->label);
  gtk_widget_show(tab->image);
  gtk_widget_show(tab->hbox);
  gtk_widget_show(tab->button);
  
  return tab;
}

const char* openldev_file_tab_get_filename (FileTab* tab)
{ return gtk_label_get_text (GTK_LABEL(tab->label));
}

void openldev_file_tab_set_filename (FileTab* tab,
                                     const char* name)
{ gtk_label_set_text (GTK_LABEL(tab->label), name);
}
