/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-output-build.h"

void erase (OutputBuild*);
void add_row (OutputBuild*, string, string);
void update_output (OutputBuild*);

OutputBuild* openldev_output_build_new ()
{
  OutputBuild *obj = new OutputBuild();
  
  obj->scrolled_window = gtk_scrolled_window_new(NULL, NULL);
  obj->build_box = gtk_tree_view_new();
  
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(obj->build_box), false);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(obj->scrolled_window),
                                         obj->build_box);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(obj->scrolled_window),
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);
  
  return obj;
}

GtkWidget* openldev_output_build_get_treeview (OutputBuild *obj)
{ return obj->build_box;
}

GtkWidget* openldev_output_build_get_scrolled_window (OutputBuild *obj)
{ return obj->scrolled_window;
}

bool openldev_output_build_run_command (OutputBuild *obj,
                                        const char* command,
                                        EnvironmentSettings *env)
{
  gchar buff[512];
  FILE *in;
  bool successful = true, autogenerate = false;
  int location;
  string text = "\0", line, comm = command;
  GtkTreeIter iter;
  
  erase (obj);
  add_row (obj, comm.substr(0, comm.find("2>&1", 0)), "#000000");
  
  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(obj->build_box));
  gtk_tree_model_get_iter (obj->model, &iter, gtk_tree_path_new_from_string("0"));
  gtk_tree_selection_select_iter (selection, &iter);
    
  // Open pipe & loop until the program is complete
  in = popen(comm.c_str(), "r");
  
  while (fgets(buff, sizeof(buff), in) && obj->continue_build)
  {
    text += buff;
    // Find the endl's & split them to be added to build_box
    location = text.find("\n",0);
    while (location != (signed int)string::npos) 
    {     
      // Check to see if autogen.sh must be run before build
      if (text.find("No targets specified",0) != string::npos)
        autogenerate = true;
        
      // Append line & erase it from the buffer
      line = text.substr(0, location);
      text.erase(0, location + 1);
      location = text.find ("\n",0);

      // Apply Error or Warning colors
      if (line.find("error: ") != string::npos) 
      {
        add_row (obj, line, env->error);
        successful = false;
      } 
      else if (line.find(": ") != string::npos)
        add_row (obj, line, env->warning);
      else
        add_row (obj, line, "#000000");
      
      // Update the build_box when there is output
      update_output(obj);
    }
  }
    
  // Close the stream, & print whether command was successful or not
  in = NULL;
  if (successful && obj->continue_build)
    add_row (obj, "Completed ... Successful", env->warning);
  else if (!obj->continue_build)
    add_row (obj, "Incomplete ... Terminated", env->error);
  else
    add_row (obj, "Completed ... Failed", env->error);
  
  GtkAdjustment *vadj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW(obj->scrolled_window));
  gtk_adjustment_set_value (vadj, vadj->upper);
  gtk_widget_set_scroll_adjustments (obj->scrolled_window, NULL, vadj);
  
  text = "\0";
  
  // Return false if the user must run autogen.sh before building & then do it
  if (autogenerate)
    return false;
  
  return true;
}

void openldev_output_build_stop_build (OutputBuild *obj)
{ obj->continue_build = false;
}

void erase (OutputBuild *obj)
{
  // Clean up build_box & get it ready to accept output
  if (gtk_tree_view_get_model(GTK_TREE_VIEW(obj->build_box)))
        gtk_tree_view_remove_column(GTK_TREE_VIEW(obj->build_box),
            gtk_tree_view_get_column(GTK_TREE_VIEW(obj->build_box), 0));                            
  obj->model = GTK_TREE_MODEL(gtk_list_store_new(2, G_TYPE_STRING, GDK_TYPE_COLOR));
  gtk_tree_view_set_model(GTK_TREE_VIEW(obj->build_box), obj->model);
  
  // Append the columns & create the cell renderer
  obj->renderer = gtk_cell_renderer_text_new();
	obj->column = gtk_tree_view_column_new_with_attributes("Search Results", 
                obj->renderer, "text", 0, "foreground-gdk", 1, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(obj->build_box), obj->column);

  obj->continue_build = true;
}

void add_row (OutputBuild *obj,
              string text, 
              string color)
{
  if (text.find ("make  all-recursive", 0) == string::npos)
  {
    GtkTreeIter iter;
    GdkColor tmp = { 0, 0, 0, 0, },
             *gcolor = gdk_color_copy(&tmp);
  
    // Add the command to the box
    gdk_color_parse(color.c_str(), gcolor);
    gtk_list_store_append(GTK_LIST_STORE(obj->model), &iter);
	  gtk_list_store_set(GTK_LIST_STORE(obj->model), &iter, 
                       0, text.c_str(), 
                       1, gcolor, -1);
  }
}

void update_output (OutputBuild *obj)
{
  GtkAdjustment *vadj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW(obj->scrolled_window));
  gtk_adjustment_set_value (vadj, vadj->upper);
  gtk_widget_set_scroll_adjustments (obj->scrolled_window, NULL, vadj);
  
  while (gtk_events_pending())
    gtk_main_iteration();
}
