/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev.h"

#define NUM_ENTRIES 45
static GtkActionEntry entries[] = {
  { "File", NULL, "_File" },
  { "Edit", NULL, "_Edit" },
  { "View", NULL, "_View" },
  { "Project", NULL, "_Project" },
  { "Build", NULL, "_Build" },
  { "Help", NULL, "_Help" },
  
  { "FileNewFile", GTK_STOCK_NEW, "_New File", "<control>n", "Create a new file", G_CALLBACK(openldev_file_new_file) },
  { "FileNewProject", NULL, "New Project", NULL, "Create a new project", G_CALLBACK(openldev_file_new_project) },
  { "FileOpen", GTK_STOCK_OPEN, "_Open", "<control>o", "Open an existing file", G_CALLBACK(openldev_file_open) },
  { "FileOpen2", GTK_STOCK_OPEN, "_Open File", NULL, "Open an existing file", G_CALLBACK(openldev_file_open) },
  { "FileSave", GTK_STOCK_SAVE, "_Save", "<control>s", "Save current file", G_CALLBACK(openldev_file_save) },
  { "FileSaveAs", GTK_STOCK_SAVE_AS, "Save _As", "<control><shift>s", "Save file by new name", G_CALLBACK(openldev_file_save_as) },
  { "FileSaveAll", NULL, "Save All", NULL, "Save all open files", G_CALLBACK(openldev_file_save_all) },
  { "FileRevert", GTK_STOCK_REVERT_TO_SAVED, "_Reload File", NULL, "Revert to saved file", G_CALLBACK(openldev_file_revert) },
  { "FilePrint", GTK_STOCK_PRINT, "_Print", "<control>p", "Print the current file", G_CALLBACK(openldev_file_print) },
  { "FilePrintPreview", GTK_STOCK_PRINT_PREVIEW, "Print Preview", "<control><shift>p", "Preview the current file", G_CALLBACK(openldev_file_print_preview) },
  { "FileClose", GTK_STOCK_CLOSE, "_Close Project", NULL, "Close current project", G_CALLBACK(openldev_file_close_project) },
  { "FileQuit", GTK_STOCK_QUIT, "_Quit", "<control>q", "Quit the application", G_CALLBACK(openldev_file_quit) },

  { "EditUndo", GTK_STOCK_UNDO, "_Undo", "<control>z", "Undo last action", G_CALLBACK(openldev_edit_undo) },
  { "EditRedo", GTK_STOCK_REDO, "_Redo", "<control>y", "Redo last undone action", G_CALLBACK(openldev_edit_redo) },
  { "EditCut", GTK_STOCK_CUT, "C_ut", "<control>x", "Cut selected text", G_CALLBACK(openldev_edit_cut) },
  { "EditCopy", GTK_STOCK_COPY, "_Copy", "<control>c", "Copy selected text", G_CALLBACK(openldev_edit_copy) },
  { "EditPaste", GTK_STOCK_PASTE, "_Paste", "<control>v", "Paste text from clipboard", G_CALLBACK(openldev_edit_paste) },
  { "EditSelectAll", NULL, "Select _All", "<control>a", "Select all text", G_CALLBACK(openldev_edit_select_all) },
  { "EditDeselect", NULL, "_Deselect", NULL, "Deselect selected text", G_CALLBACK(openldev_edit_deselect) },
  { "EditFind", GTK_STOCK_FIND, "_Find", "<control>f", "Open the Find toolbar", G_CALLBACK(openldev_edit_find) },
  { "EditFindNext", GTK_STOCK_FIND, "Find _Next", "F3", "Find next", G_CALLBACK(openldev_edit_find_next) },
  { "EditGotoLine", GTK_STOCK_JUMP_TO, "_Goto Line", "<control>g", "Open Goto Line toolbar", G_CALLBACK(openldev_edit_goto_line) },
  { "EditSettings", GTK_STOCK_PREFERENCES, "Preferences", NULL, "Edit Environment settings", G_CALLBACK(openldev_edit_preferences) },
  
  { "ViewFiles", GTK_STOCK_JUSTIFY_LEFT, "_Project Files", NULL, "Show or hide project files", G_CALLBACK(openldev_view_browser) },
  { "ViewOutput", GTK_STOCK_JUSTIFY_LEFT, "_Build Output", NULL, "Show or hide build output", G_CALLBACK(openldev_view_output) },
  
  { "ProjectFiles", GTK_STOCK_PROPERTIES, "_Makefile Editor", NULL, "Edit your project Makefile.am's", G_CALLBACK(openldev_project_makefile_editor) },
  { "ProjectConfigure", GTK_STOCK_PREFERENCES, "_Configure Project", NULL, "Edit project properties", G_CALLBACK(openldev_project_preferences) },
  { "ProjectInfo", NULL, "_Project Info", NULL, "View project info", G_CALLBACK(openldev_project_info) },
  
  { "BuildCompile", "COMPILE", "_Compile", "F9", "Compile current file", G_CALLBACK(openldev_build_compile) },
  { "BuildBuild", "BUILD_SM", "_Build", "F11", "Build project tree", G_CALLBACK(openldev_build_project) },
  { "BuildBuild2", "BUILD_LG", "_Build", NULL, "Build project tree", G_CALLBACK(openldev_build_project) },
  { "BuildExecute", GTK_STOCK_EXECUTE, "_Execute", "F4", "Execute project", G_CALLBACK(openldev_build_execute) },
  { "BuildAutogen", GTK_STOCK_CONVERT, "_Auto Generate", NULL, "Auto generate and configure", G_CALLBACK(openldev_build_auto_generate) },
  { "BuildConfigure", NULL, "Con_figure", NULL, "Run configure script", G_CALLBACK(openldev_build_configure) },
  { "BuildClean", GTK_STOCK_CLEAR, "_Make Clean", NULL, "Clean distribution tree", G_CALLBACK(openldev_build_clean) },
  { "BuildStop", GTK_STOCK_STOP, "_Stop", "F12", "Stop current action", G_CALLBACK(openldev_build_stop) },
  
  { "HelpDocs", GTK_STOCK_HELP, "_Documentation", "F1", "View project documentation", G_CALLBACK(openldev_help_documentation) },
  { "HelpBugs", NULL, "_Report Bugs", NULL, "Report bugs", G_CALLBACK(openldev_help_report_bug) },
  { "HelpAbout", GTK_STOCK_ABOUT, "_About", "<alt>a", "About OpenLDev", G_CALLBACK(openldev_help_about) }
};

#define NUM_ENTRIES_TOGGLE 3
static GtkToggleActionEntry toggle_entries[] = {
  { "ViewToolbar", NULL, "_Toolbar", NULL, "Show or hide toolbar", G_CALLBACK(openldev_view_toolbar), true },
  { "ViewStatusBar", NULL, "_Status Bar", NULL, "Show or hide status bar", G_CALLBACK(openldev_view_statusbar), true },
  { "ViewLineNumbering", NULL, "_Line Numbering", NULL, "Show or hide line numbers", G_CALLBACK(openldev_view_line_numbering), true }
};

#define NUM_MENUITEMS 20
gchar* menuitems[] = { "/MenuBar/File/FileSave", "/MenuBar/File/FileSaveAs", "/MenuBar/File/FileSaveAll", 
       "/MenuBar/File/FileRevert", "/MenuBar/File/FilePrint", "/MenuBar/File/FilePrintPreview", 
       "/MenuBar/Edit/EditUndo", "/MenuBar/Edit/EditRedo", "/MenuBar/Edit/EditCut", "/MenuBar/Edit/EditCopy", 
       "/MenuBar/Edit/EditPaste", "/MenuBar/Edit/EditSelectAll", "/MenuBar/Edit/EditDeselect",
       "/ToolBar/FileSave", "/ToolBar/FileRevert", "/ToolBar/EditUndo", "/ToolBar/EditRedo",
       "/ToolBar/EditCut", "/ToolBar/EditCopy", "/ToolBar/EditPaste" };

// Initialization Functions
void openldev_initialize_objects (OpenLDev*);
void openldev_populate_window    (OpenLDev*);
void openldev_create_icon_factory ();
void openldev_create_menus_and_toolbar (OpenLDev*);
void openldev_add_new_stock_item (GtkIconFactory*, string, string, string);
void openldev_connect_signals (OpenLDev*);

// Signal Handler declarations
bool openldev_on_auto_save (gpointer);
bool openldev_on_reset_status_bar (gpointer);
bool openldev_on_manage_menu_items (gpointer);
bool openldev_on_button_press_event_treeview (GtkWidget*, GdkEventButton*, gpointer);
bool openldev_on_build_box_selected (GtkWidget*, GdkEventButton*, gpointer);
void openldev_on_resize (GtkWidget*, GdkEvent*, gpointer);
void openldev_on_resize_pane (GtkWidget*, GdkEvent*, gpointer);
void openldev_on_close_toolbars (GtkWidget*, OpenLDev*);
void openldev_on_goto_line (GtkWidget*, gpointer);
void openldev_on_find (GtkWidget*, gpointer);
void openldev_on_find_and_replace (GtkWidget*, gpointer);
bool openldev_on_search_box_selected (GtkWidget*, GdkEventButton*, gpointer);

// Utility Functions
bool openldev_is_error_or_warning (string);
int openldev_get_file_name_and_line (string&);

OpenLDev* openldev_new_with_window (GtkWidget *win)
{
  OpenLDev *openldev = new OpenLDev;
  
  // Set properties of main window
  openldev->window = win;
  string title = string("OpenLDev ") + VERSION;
  gtk_window_set_title (GTK_WINDOW(openldev->window), title.c_str());
  gtk_container_set_border_width (GTK_CONTAINER(openldev->window), 0);
  title = "\0";
  
  // Create settings objects
  openldev->project = openldev_project_settings_new("");
  openldev->env = openldev_environment_settings_new();
  openldev->editor = openldev_editor_new();
  openldev->ptemplate = openldev_project_template_new("console-c");
  openldev->templatelist = openldev_template_list_new();
  
  // Call functions to add main window widgets & show them all
  gtk_window_set_default_size (GTK_WINDOW(openldev->window), 
                               openldev->env->width,
                               openldev->env->height);
  
  openldev_initialize_objects (openldev);
  gtk_notebook_set_current_page (GTK_NOTEBOOK(openldev->output->notebook), 0);
  
  if (openldev->env->auto_save > 0)
  {
    gint time;
    time = openldev->env->auto_save * 1000 * 60;
    g_timeout_add (time, GSourceFunc(openldev_on_auto_save), (gpointer)openldev);
  }
  
  g_timeout_add (2000, GSourceFunc(openldev_on_reset_status_bar), (gpointer)openldev);
  g_timeout_add (2000, GSourceFunc(openldev_on_manage_menu_items), (gpointer)openldev);
  
  // Load the last project or just set the directory to the home directory
  if (openldev->env->last_project != "EMPTY" && 
      openldev->env->last_project != "" && 
      openldev->env->auto_load)
    openldev_open_project (openldev, openldev->env->last_project.c_str());
  
  return openldev;
}

void openldev_initialize_objects (OpenLDev *openldev)
{
  openldev->vpaned = gtk_vpaned_new();
  openldev->hpaned = gtk_hpaned_new();
  openldev->vbox = gtk_vbox_new(false, 0);
  openldev->hbox = gtk_hbox_new(false, 0);
  openldev->statusbar = openldev_status_bar_new ();
  openldev->output = openldev_output_manager_new ();
  openldev->notebook = openldev_file_manager_new ();
  openldev->browser = openldev_browser_new ();
  openldev->findtoolbar = openldev_find_toolbar_new ();
  openldev->gototoolbar = openldev_goto_toolbar_new ();
  openldev->menus = openldev_menu_handle_new ();
  
  gtk_widget_set_size_request (openldev->browser->files->scrolled_window, 
                               openldev->env->browser_width, 0);
  gtk_widget_set_size_request (openldev->output->terminal->terminal, 0, 
                               openldev->env->outputmanager_height);
  openldev_browser_project_populate (openldev->browser->files, "NULL");
  
  // Add all of the widgets & show them
  openldev_populate_window (openldev);
  openldev_connect_signals (openldev);
  gtk_widget_show_all (openldev->window);
  
  openldev_find_toolbar_show_error (openldev->findtoolbar, false);
  openldev_goto_toolbar_show_error (openldev->gototoolbar, false);
  gtk_widget_hide (openldev->findtoolbar->hbox);
  gtk_widget_hide (openldev->gototoolbar->hbox);
}

void openldev_populate_window (OpenLDev *openldev)
{
  openldev_create_menus_and_toolbar (openldev);
  
  // Add menu & toolbar to main vbox
  gtk_box_pack_start (GTK_BOX(openldev->vbox), openldev->menus->menubar, false, false, 0);
  gtk_box_pack_start (GTK_BOX(openldev->vbox), openldev->menus->toolbar, false, false, 0);
  gtk_paned_pack1 (GTK_PANED(openldev->hpaned), openldev->browser->notebook, false, true);
  gtk_paned_pack2 (GTK_PANED(openldev->hpaned), openldev->notebook->notebook, true, true);
  gtk_paned_pack1 (GTK_PANED(openldev->vpaned), openldev->hpaned, true, true);
  gtk_paned_pack2 (GTK_PANED(openldev->vpaned), openldev->output->notebook, false, true);

  // Add scrolled_win3 to the second vbox and finish off with the statusbar
  gtk_box_pack_start (GTK_BOX(openldev->vbox), openldev->vpaned, true, true, 0);
  gtk_box_pack_start (GTK_BOX(openldev->vbox), openldev->gototoolbar->hbox, false, false, 0);
  gtk_box_pack_start (GTK_BOX(openldev->vbox), openldev->findtoolbar->hbox, false, false, 0);
  gtk_box_pack_start (GTK_BOX(openldev->vbox), openldev->statusbar->appbar, false, false, 0);

  // Add v' & h' boxes to the main window
  gtk_container_add (GTK_CONTAINER(openldev->window), openldev->vbox);
}

void openldev_connect_signals (OpenLDev *openldev)
{
  g_signal_connect (G_OBJECT(openldev->window), "event-after",
                    G_CALLBACK(openldev_on_resize), (gpointer)openldev);
  g_signal_connect (G_OBJECT(openldev->hpaned), "event-after",
                    G_CALLBACK(openldev_on_resize_pane), (gpointer)openldev);
  g_signal_connect (G_OBJECT(openldev->vpaned), "event-after",
                    G_CALLBACK(openldev_on_resize_pane), (gpointer)openldev);
  g_signal_connect (G_OBJECT(openldev->browser->files->treeview), "button-press-event",
                    G_CALLBACK(openldev_on_button_press_event_treeview), (gpointer)openldev);
  g_signal_connect (G_OBJECT(openldev->output->search->search_box), "button-press-event", 
                    G_CALLBACK(openldev_on_search_box_selected), (gpointer)openldev);
  g_signal_connect (G_OBJECT(openldev->output->build->build_box), "button-press-event",
                    G_CALLBACK(openldev_on_build_box_selected), (gpointer)openldev);
  g_signal_connect (G_OBJECT(openldev->gototoolbar->button_close), "clicked",
                    G_CALLBACK(openldev_on_close_toolbars), (gpointer)openldev);
  g_signal_connect (G_OBJECT(openldev->gototoolbar->button_goto_line), "clicked",
                    G_CALLBACK(openldev_on_goto_line), (gpointer)openldev);
  g_signal_connect (G_OBJECT(openldev->findtoolbar->button_close), "clicked",
                    G_CALLBACK(openldev_on_close_toolbars), (gpointer)openldev);
  g_signal_connect (G_OBJECT(openldev->findtoolbar->button_find), "clicked",
                    G_CALLBACK(openldev_on_find), (gpointer)openldev);
  g_signal_connect (G_OBJECT(openldev->findtoolbar->button_replace), "clicked",
                    G_CALLBACK(openldev_on_find_and_replace), (gpointer)openldev);
}

void openldev_populate_treeview (OpenLDev *openldev)
{ openldev_browser_project_populate(openldev->browser->files, openldev->project->location.c_str());
}

// This function creates a new icon factory that adds the stock menu/toolbar
// items for the compile, build, project files and about commands
void openldev_create_icon_factory ()
{
  GtkIconFactory *icon_factory = gtk_icon_factory_new();;
  
  // Add new stock menu items
  openldev_add_new_stock_item(icon_factory, "Compile", "COMPILE", string(INSTALL_PREFIX) + "/share/pixmaps/openldev/compile_sm.png");
  openldev_add_new_stock_item(icon_factory, "Build", "BUILD_SM", string(INSTALL_PREFIX) + "/share/pixmaps/openldev/build_sm.png");
  openldev_add_new_stock_item(icon_factory, "Build", "BUILD_LG", string(INSTALL_PREFIX) + "/share/pixmaps/openldev/build_lg.png");
  
  // Add items to default
  gtk_icon_factory_add_default (icon_factory);
  icon_factory = NULL;
}

// Create new stock menu item. This function receives the icon factory, the
// name of the item, the id of the item, and the filename of the icon.
void openldev_add_new_stock_item (GtkIconFactory *icon_factory, 
                                  string name,
                                  string id, 
                                  string filename)
{
  GtkIconSource *icon_source = gtk_icon_source_new();
  GtkIconSet *icon = gtk_icon_set_new();
  
  // Set the properties of the stock menu/toolbar item
  gtk_icon_source_set_icon_name (icon_source, name.c_str());
  gtk_icon_source_set_filename (icon_source, filename.c_str());
  gtk_icon_set_add_source (icon, icon_source);
  gtk_icon_factory_add (icon_factory, id.c_str(), icon);
  
  name = "\0";
  id = "\0";
  filename = "\0";
  icon_source = NULL;
  icon = NULL;
}

void openldev_create_menus_and_toolbar (OpenLDev *openldev)
{
  // Create menu actions and icons
  openldev_create_icon_factory();

  gtk_action_group_add_actions (openldev->menus->group, entries, NUM_ENTRIES, (gpointer)openldev);
  gtk_action_group_add_toggle_actions (openldev->menus->group, toggle_entries, NUM_ENTRIES_TOGGLE, (gpointer)openldev);
  
  string ui_file = string(INSTALL_PREFIX) + "/share/openldev/menus/mainmenu.ui",
         ui_file2 = string(INSTALL_PREFIX) + "/share/openldev/menus/toolbar.ui",
         ui_file3 = string(INSTALL_PREFIX) + "/share/openldev/menus/popup_treeview.ui";
  GError *uierror;
  
  if (!gtk_ui_manager_add_ui_from_file (openldev->menus->uimanager, ui_file.c_str(), &uierror))
    openldev_error ("Cannot load file: mainmenu.ui");
  if (!gtk_ui_manager_add_ui_from_file (openldev->menus->uimanager, ui_file2.c_str(), &uierror))
    openldev_error ("Cannot load file: toolbar.ui");
  if (!gtk_ui_manager_add_ui_from_file (openldev->menus->uimanager, ui_file3.c_str(), &uierror))
    openldev_error ("Cannot load file: popup_treeview.ui");
  
  gtk_ui_manager_insert_action_group (openldev->menus->uimanager, openldev->menus->group, 0);
  gtk_window_add_accel_group (GTK_WINDOW(openldev->window), 
                              gtk_ui_manager_get_accel_group(openldev->menus->uimanager));
  
  openldev->menus->menubar = gtk_ui_manager_get_widget (openldev->menus->uimanager, "/MenuBar");
  openldev->menus->toolbar = gtk_ui_manager_get_widget (openldev->menus->uimanager, "/ToolBar");
  openldev->menus->popup = gtk_ui_manager_get_widget (openldev->menus->uimanager, "/TreeViewMenu");
  
  GtkWidget *linenums, *close, *project, *build;
  
  linenums = gtk_ui_manager_get_widget (openldev->menus->uimanager, "/MenuBar/View/ViewLineNumbering");
  close = gtk_ui_manager_get_widget (openldev->menus->uimanager, "/MenuBar/File/FileClose");
  project = gtk_ui_manager_get_widget (openldev->menus->uimanager, "/MenuBar/Project");
  build = gtk_ui_manager_get_widget (openldev->menus->uimanager, "/MenuBar/Build");
    
  gtk_widget_set_sensitive (close, false);
  gtk_widget_set_sensitive (project, false);
  gtk_widget_set_sensitive (build, false);
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(linenums),  openldev->env->line_numbers);

  gtk_toolbar_set_style (GTK_TOOLBAR(openldev->menus->toolbar), GTK_TOOLBAR_ICONS);
  gtk_toolbar_set_show_arrow (GTK_TOOLBAR(openldev->menus->toolbar), false);
  
  int i;
  for (i=0; i < NUM_ENTRIES; i++)
  {
    entries[i].name = NULL;
    entries[i].stock_id = NULL;
    entries[i].label = NULL;
    entries[i].accelerator = NULL;
    entries[i].tooltip = NULL;
  }
  for (i=0; i < NUM_ENTRIES_TOGGLE; i++)
  {
    toggle_entries[i].name = NULL;
    toggle_entries[i].stock_id = NULL;
    toggle_entries[i].label = NULL;
    toggle_entries[i].accelerator = NULL;
    toggle_entries[i].tooltip = NULL;
  }
  
  ui_file = "\0";
  ui_file2 = "\0";
  ui_file3 = "\0";
}

// Throw an error dialog with the specified message
void openldev_error (const char *message)
{
  GtkWidget *dialog;
  dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, 
                                   GTK_BUTTONS_CLOSE, message);
  int result = gtk_dialog_run (GTK_DIALOG(dialog));
  
  switch (result)
  {
    default:
      gtk_widget_destroy(dialog);
  }
}

// Show a question dialog with the specified message. Return the answer the
// user choses. This function handles creating the dialog, setting the properties
// and then deleting it and returning the answer.
bool openldev_question (const char *question)
{
  GtkWidget *dialog = gtk_dialog_new(),
            *yes = gtk_button_new_from_stock("gtk-yes"),
            *no = gtk_button_new_from_stock("gtk-no"),
            *image = gtk_image_new_from_stock("gtk-dialog-warning", GTK_ICON_SIZE_DIALOG),
            *label = gtk_label_new(question),
            *hbox = gtk_hbox_new(false, 10);
  
  gtk_container_set_border_width (GTK_CONTAINER(hbox), 10);
  gtk_box_pack_start (GTK_BOX(hbox), image, false, false, 0);
  gtk_box_pack_start (GTK_BOX(hbox), label, false, false, 0);
  
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox),
                      hbox, TRUE, TRUE, 0);
  gtk_box_set_spacing (GTK_BOX(GTK_DIALOG(dialog)->vbox), 7);
  gtk_window_set_title (GTK_WINDOW(dialog), "Question");  
  gtk_window_set_modal (GTK_WINDOW(dialog), true);
  gtk_window_set_resizable (GTK_WINDOW(dialog), false);
  gtk_dialog_set_has_separator (GTK_DIALOG(dialog), true);
  gtk_dialog_add_action_widget (GTK_DIALOG(dialog), no, -9);
  gtk_dialog_add_action_widget (GTK_DIALOG(dialog), yes, -8);
  
  gtk_widget_show_all(dialog);
  
  switch (gtk_dialog_run(GTK_DIALOG(dialog)))
  {
    case -8:
    {
      gtk_widget_destroy(dialog);
      return true;
      break;
    }
    default:
    {
      gtk_widget_destroy(dialog);
      return false;
      break;
    }
  }
}

// Write the text to the file. This function receives the filename and the file
// text. The text is then written to the file located at the filename.
void openldev_write_file (string filename, 
                          string text)
{
  ofstream fout(filename.c_str());
  fout << text;
  fout.close();
}

// Read in the text from a file. This function receives a filename and returns
// the text string of the file.
string openldev_read_file (string filename)
{
  if (!g_file_test(filename.c_str(), G_FILE_TEST_EXISTS))
    return string("");
  
  string text;
  string line;
  ifstream fin(filename.c_str());
  while (getline(fin, line))
    text += g_locale_to_utf8 (line.c_str(), -1, NULL, NULL, NULL) + string("\n");
  
  return text;
}

// Quit the program
void openldev_file_quit (GtkMenuItem* menuitem, 
                         gpointer data) 
{ 
  OpenLDev *openldev = (OpenLDev*) data;
  bool is_modified = false;
  
  for (int i=0; i < openldev->notebook->tab_count; i++)
  {
    string label = gtk_label_get_text (GTK_LABEL(openldev->notebook->tabs[i]->label));
    if (label.find("*", 0) != string::npos)
      is_modified = true;
    label = "\0";
  }
  
  if (!is_modified)
    gtk_widget_destroy (openldev->window);
  else if (openldev_file_manager_question("You have unsaved files! Are you.\nsure you want to close?"))
    gtk_widget_destroy (openldev->window);

}

bool openldev_on_auto_save (gpointer data)
{
  openldev_menu_file_save_all ((OpenLDev*)data);
  return true;
}

// When the window is resized, this checks the width and height and stores
// them so the same size can be restored on restart.
void openldev_on_resize (GtkWidget *widget,
                         GdkEvent *event,
                         gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (event->type == GDK_CONFIGURE)
  {
    // Get the current height & width and store them
    int width=0, height=0;
    width = openldev->window->allocation.width;
    height = openldev->window->allocation.height;
    openldev_environment_set_width (openldev->env, width);
    openldev_environment_set_height (openldev->env, height);
  }
}

// When one of the panes is resized, this checks the width and height of each
// pane so the same sizes can be restored on restart.
void openldev_on_resize_pane (GtkWidget *widget,
                              GdkEvent *event,
                              gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (event->type == GDK_MOTION_NOTIFY)
  {   
    // Get the Project File & Output panes width/height & store them
    openldev_environment_set_browser_width (openldev->env, openldev->browser->files->scrolled_window->allocation.width);
    openldev_environment_set_outputmanager_height (openldev->env, openldev->output->build->scrolled_window->allocation.height);
  }
}

// Catch button press event & open popup menu or handle selection. This function
// receives the button event. If it is a right click, then the popup menu is
// shown. If it is a double click, the selected file is opened
bool openldev_on_button_press_event_treeview (GtkWidget *widget,
                                              GdkEventButton *event,
                                              gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if((event->type == GDK_BUTTON_PRESS) && (event->button == 3))  
    gtk_menu_popup (GTK_MENU(openldev->menus->popup), 
                    NULL, NULL, NULL, data, 0, event->time);
  else if (event->type == GDK_2BUTTON_PRESS)
  {
    GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
    GtkTreeModel *model;
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
      gchar *fn;
      gtk_tree_model_get(model, &iter, 1, &fn, -1);
   
      // If the text is not a group title, open it
      string file = openldev->browser->files->base
                  + openldev->browser->files->current + fn;
      
      if (string(fn) == "..")
        openldev_browser_project_move_up (openldev->browser->files);
      else if (g_file_test(file.c_str(), G_FILE_TEST_IS_DIR))
        openldev_browser_project_move (openldev->browser->files, fn);
      else
        openldev_open_file (openldev, file.c_str());
      
      file = "\0";
      fn = "\0";
    }
  }
  else
    return false;
  
  return true;
}

// Catch the build box selection & do action. This function receives the button
// event. If it is a double click, get the text of the selection and check if
// it is an error or warning. If it is, then the file containing the error or
// warning is opened. The cursor is then moved line of the error or warning
bool openldev_on_build_box_selected (GtkWidget *widget, 
                                     GdkEventButton *event, 
                                     gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  
  if (event->type == GDK_2BUTTON_PRESS)
  {
    GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
    GtkTreeModel *model;
    GtkTreePath *path = gtk_tree_path_new ();
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
      string file, dstr;
      gchar *output, *dir;
      int linenumber;
      gtk_tree_model_get (model, &iter, 0, &output, -1);
      
      // It the output is an error or a warning..
      if (openldev_is_error_or_warning(output))
      {
        file = string(output);
        linenumber = openldev_get_file_name_and_line(file) - 1;
        
        do
        {
          path = gtk_tree_model_get_path (model, &iter);
          gtk_tree_path_prev (path);
          gtk_tree_model_get_iter (model, &iter, path);
          gtk_tree_model_get (model, &iter, 0, &dir, -1);
          dstr = string(dir);
        } while (dstr.find("make[", 0) == string::npos);
        
        dstr = dir;
        dstr.erase (0, dstr.find ("Entering", 0) + 20);
        dstr = dstr.substr (0, dstr.find ("'", 0));

        openldev_open_file (openldev, string(dstr + "/" + file).c_str());
        
        // Move the cursor to the line of the error & scroll to the cursor
        GtkTextBuffer *buffer = GTK_TEXT_BUFFER(openldev_file_manager_get_textbuffer(openldev->notebook));
        GtkTextIter iter;

        gtk_text_buffer_get_iter_at_line(buffer, &iter, linenumber);
        gtk_text_buffer_move_mark(buffer, gtk_text_buffer_get_insert(buffer), &iter);
        gtk_text_buffer_move_mark(buffer, gtk_text_buffer_get_selection_bound(buffer), &iter);
        openldev_file_manager_scroll_to_mark (openldev->notebook);
        gtk_widget_grab_focus(openldev_file_manager_get_textview(openldev->notebook));
      }
    }
    
    return true;
  }
  
  return false;
}

// Catch the search box selection & do action. This function receives the button
// event. If it is a double click, get the text of the selection and then the 
// file containing the search text is opened. The cursor is then moved line of 
// the search result
bool openldev_on_search_box_selected (GtkWidget *widget,
                                      GdkEventButton *event,
                                      gpointer data)
{
  OpenLDev *openldev = (OpenLDev*)data;
  
  if (event->type == GDK_2BUTTON_PRESS)
  {
    GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
    GtkTreeModel *model;
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
      string file;
      gchar *output;
      int linenumber;
      gtk_tree_model_get (model, &iter, 0, &output, -1);
      
      // Open the file the error was found in
      file = string(output);
      linenumber = openldev_get_file_name_and_line(file) - 1;
      file.insert (0, openldev->project->location);
      openldev_open_file (openldev, file.c_str());
      
      // Move the cursor to the line of the error & scroll to the cursor
      GtkTextIter iter;
      GtkTextBuffer *buffer = GTK_TEXT_BUFFER (openldev_file_manager_get_textbuffer(openldev->notebook));
      
      gtk_text_buffer_get_iter_at_line (buffer, &iter, linenumber);
      gtk_text_buffer_move_mark (buffer, gtk_text_buffer_get_insert(buffer), &iter);
      gtk_text_buffer_move_mark (buffer, gtk_text_buffer_get_selection_bound(buffer), &iter);
      openldev_file_manager_scroll_to_mark (openldev->notebook);
      gtk_widget_grab_focus (openldev_file_manager_get_textview(openldev->notebook));
    }
    
    return true;
  }
  
  return false;
}

// Check if the output is an error or warning. If it only has two colons in
// close proximity of eachother in the beginning, then it is an error
// or a warning.
bool openldev_is_error_or_warning (string output)
{
  // Find :'s (It is an error or warning if 2 :'s are found close together
  int location  = output.find(":", 0);
  int location2 = output.find(":", location+1);
  
  // Check if :'s were found & return if they were in proper format
  if (location != (signed int)string::npos)
    if (location2 != (signed int)string::npos && (location2-location < 5))
      return true;
  
  return false;
}

// Get file name and line of the error or warning. This function parses the
// error or warning sent by the output string. Output is then converted into
// only the short filename containing the error & the line number of the
// error or warning is returned.
int openldev_get_file_name_and_line (string &output)
{
  string line;
  int location = output.find(":", 0);
  
  // Get the line number & delete it out so only filename is left
  line = output.substr(location+1, output.length()-location-1);
  output.erase(location, output.length()-location);
  
  // Erase all except for the line number
  location = line.find(":", 0);
  line.erase(location, line.length()-location);
  
  // Return the line number
  return atol(line.c_str());
}

// Close both toolbars. The find toolbar and the goto line toolbar.
void openldev_on_close_toolbars (GtkWidget *widget, 
                                 OpenLDev *openldev)
{
  gtk_widget_hide (openldev->gototoolbar->hbox);
  gtk_widget_hide (openldev->findtoolbar->hbox);
}

void openldev_on_goto_line (GtkWidget *widget,
                            gpointer data)
{ openldev_goto_to_line_number ((OpenLDev*)data, -1);
}

void openldev_on_find (GtkWidget *widget,
                       gpointer data)
{ openldev_find_next ((OpenLDev*)data);
}

void openldev_on_find_and_replace (GtkWidget *widget,
                                   gpointer data)
{ openldev_find_and_replace ((OpenLDev*)data);
}

bool openldev_on_reset_status_bar (gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  string message = "\0";
  
  if (!openldev->project->name.empty())
  {
    message += "Project: " + openldev->project->name + "     ";
    message += "Version: " + openldev->project->version + "     ";
  }
  
  if (openldev_file_manager_get_file_open (openldev->notebook))
  {
    GtkTextBuffer *buffer = GTK_TEXT_BUFFER(openldev_file_manager_get_textbuffer(openldev->notebook));
    GtkTextIter insert;
    gtk_text_buffer_get_iter_at_mark (buffer, &insert, gtk_text_buffer_get_insert(buffer));
  
    char vertical[5], horizontal[5];
    g_snprintf(vertical, sizeof(vertical), "%d", gtk_text_iter_get_line(&insert)+1);
    g_snprintf(horizontal, sizeof(horizontal), "%d", gtk_text_iter_get_line_offset(&insert)+1);
  
    message += "Line: " + string(vertical) + "     ";
    message += "Column: " + string(horizontal);
  }
  
  openldev_status_bar_set_message (openldev->statusbar, message.c_str());
  message = "\0";
  return true;
}

bool openldev_on_manage_menu_items (gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  GtkWidget *item;
  bool show = openldev_file_manager_get_file_open (openldev->notebook);
  item = gtk_ui_manager_get_widget (openldev->menus->uimanager, menuitems[0]);
  
  if (!(show == GTK_WIDGET_SENSITIVE(item)))
  {
    for (unsigned int i=0; i < NUM_MENUITEMS; i++)
    {
      item = gtk_ui_manager_get_widget (openldev->menus->uimanager, menuitems[i]);
      gtk_widget_set_sensitive (item, show);
    }
  }
  return true;
}
