/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef OPENLDEV_MAIN_H
#define OPENLDEV_MAIN_H

#include "openldev-widgets/openldev-browser.h"
#include "openldev-widgets/openldev-goto-line-toolbar.h"
#include "openldev-widgets/openldev-find-toolbar.h"
#include "openldev-widgets/openldev-menu.h"
#include "openldev-widgets/openldev-file-manager.h"
#include "openldev-widgets/openldev-output-manager.h"
#include "openldev-widgets/openldev-status-bar.h"
#include "openldev-settings/openldev-editor.h"
#include "openldev-settings/openldev-environment.h"
#include "openldev-settings/openldev-project.h"
#include "openldev-settings/openldev-project-template.h"
#include "openldev-settings/openldev-template-list.h"

#include <sys/stat.h>

#define VERSION "0.5.0"

typedef struct _OpenLDev  OpenLDev;

struct _OpenLDev
{  
  // Class Widgets
  Browser         *browser;
  FileManager     *notebook;
  FindToolbar     *findtoolbar;
  GotoLineToolbar *gototoolbar;
  MenuHandle      *menus;
  OutputManager   *output;
  StatusBar       *statusbar;

  // Gtk Widgets
  GtkWidget *hbox;
  GtkWidget *hpaned;
  GtkWidget *vbox;
  GtkWidget *vpaned;
  GtkWidget *window;
  
  // Environment & project settings
  EditorProperties    *editor;
  EnvironmentSettings *env;
  ProjectSettings     *project;
  ProjectTemplate     *ptemplate;
  TemplateList        *templatelist;
};

OpenLDev* openldev_new_with_window (GtkWidget*);

// Utility Functions
void   openldev_error      (const char*);
bool   openldev_question   (const char*);
void   openldev_write_file (string, string);
string openldev_read_file  (string);

// File Management Functions
void openldev_open_file    (OpenLDev*, const char*);
void openldev_open_project (OpenLDev*, const char*);
void openldev_populate_treeview (OpenLDev*);

// Toolbar Functions
void openldev_goto_to_line_number (OpenLDev*, int);
void openldev_find_next           (OpenLDev*);
void openldev_find_and_replace    (OpenLDev*);

// Internal OpenLDev menu handlers
void openldev_file_close_project (GtkMenuItem*, gpointer);
void openldev_file_new_file      (GtkMenuItem*, gpointer);
void openldev_file_new_project   (GtkMenuItem*, gpointer);
void openldev_file_open          (GtkMenuItem*, gpointer);
void openldev_file_print         (GtkMenuItem*, gpointer);
void openldev_file_print_preview (GtkMenuItem*, gpointer);
void openldev_file_revert        (GtkMenuItem*, gpointer);
void openldev_file_save          (GtkMenuItem*, gpointer);
void openldev_file_save_all      (GtkMenuItem*, gpointer);
void openldev_file_save_as       (GtkMenuItem*, gpointer);
void openldev_file_quit          (GtkMenuItem*, gpointer);

void openldev_edit_copy        (GtkMenuItem*, gpointer);
void openldev_edit_cut         (GtkMenuItem*, gpointer);
void openldev_edit_deselect    (GtkMenuItem*, gpointer);
void openldev_edit_find        (GtkMenuItem*, gpointer);
void openldev_edit_find_next   (GtkMenuItem*, gpointer);
void openldev_edit_goto_line   (GtkMenuItem*, gpointer);
void openldev_edit_paste       (GtkMenuItem*, gpointer);
void openldev_edit_preferences (GtkMenuItem*, gpointer);
void openldev_edit_redo        (GtkMenuItem*, gpointer);
void openldev_edit_select_all  (GtkMenuItem*, gpointer);
void openldev_edit_undo        (GtkMenuItem*, gpointer);

void openldev_project_info            (GtkMenuItem*, gpointer);
void openldev_project_makefile_editor (GtkMenuItem*, gpointer);
void openldev_project_preferences     (GtkMenuItem*, gpointer);

void openldev_build_auto_generate (GtkMenuItem*, gpointer);
void openldev_build_clean         (GtkMenuItem*, gpointer);
void openldev_build_compile       (GtkMenuItem*, gpointer);
void openldev_build_configure     (GtkMenuItem*, gpointer);
void openldev_build_execute       (GtkMenuItem*, gpointer);
void openldev_build_project       (GtkMenuItem*, gpointer);
void openldev_build_stop          (GtkMenuItem*, gpointer);

void openldev_view_browser        (GtkMenuItem*, gpointer);
void openldev_view_line_numbering (GtkMenuItem*, gpointer);
void openldev_view_output         (GtkMenuItem*, gpointer);
void openldev_view_statusbar      (GtkMenuItem*, gpointer);
void openldev_view_toolbar        (GtkMenuItem*, gpointer);

void openldev_help_about         (GtkMenuItem *menuitem, gpointer data);
void openldev_help_documentation (GtkMenuItem *menuitem, gpointer data);
void openldev_help_report_bug    (GtkMenuItem *menuitem, gpointer data);

// API menu handlers
void opendlev_menu_file_close_project (OpenLDev*);
void opendlev_menu_file_new_file      (OpenLDev*);
void openldev_menu_file_new_project   (OpenLDev*);
void openldev_menu_file_open          (OpenLDev*);
void openldev_menu_file_print         (OpenLDev*);
void openldev_menu_file_print_preview (OpenLDev*);
void openldev_menu_file_quit          (OpenLDev*);
void openldev_menu_file_revert        (OpenLDev*);
void openldev_menu_file_save          (OpenLDev*);
void openldev_menu_file_save_all      (OpenLDev*);
void openldev_menu_file_save_as       (OpenLDev*);

void openldev_menu_edit_copy        (OpenLDev*);
void openldev_menu_edit_cut         (OpenLDev*);
void openldev_menu_edit_deselect    (OpenLDev*);
void openldev_menu_edit_find        (OpenLDev*);
void openldev_menu_edit_find_next   (OpenLDev*);
void openldev_menu_edit_goto_line   (OpenLDev*);
void openldev_menu_edit_paste       (OpenLDev*);
void openldev_menu_edit_preferences (OpenLDev*);
void openldev_menu_edit_redo        (OpenLDev*);
void openldev_menu_edit_select_all  (OpenLDev*);
void openldev_menu_edit_undo        (OpenLDev*);

void openldev_menu_project_info            (OpenLDev*);
void openldev_menu_project_makefile_editor (OpenLDev*);
void openldev_menu_project_preferences     (OpenLDev*);

void openldev_menu_build_auto_generate (OpenLDev*);
void openldev_menu_build_project       (OpenLDev*);
void openldev_menu_build_clean         (OpenLDev*);
void openldev_menu_build_compile       (OpenLDev*);
void openldev_menu_build_configure     (OpenLDev*);
void openldev_menu_build_execute       (OpenLDev*);
void openldev_menu_build_stop          (OpenLDev*);

void openldev_menu_view_browser        (OpenLDev*);
void openldev_menu_view_line_numbering (OpenLDev*);
void openldev_menu_view_output         (OpenLDev*);
void openldev_menu_view_statusbar      (OpenLDev*);
void openldev_menu_view_toolbar        (OpenLDev*);

void openldev_menu_help_documentation (OpenLDev*);
void openldev_menu_help_report_bug    (OpenLDev*);
void openldev_menu_help_about         (OpenLDev*);

#endif
